%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          Computes the C-matrix and its asymptotic standard     %
%          errors                                                %
%                                                                %
%          Function: cmat                                        %
%                                                                %
%          Input:                                                %
%                       Psi: nxnq matrix                         %
%                       alpha: nxr matrix                        %
%                       beta: n+q1+dcointxr matrix with          %
%                           cointegration vectors                %
%                       epsilon: nxT matrix with residuals       %
%                       Z1: nqxT matrix                          %
%                       Z2: nrxT matrix                          %
%                       p: number of lags                        %
%                       d: number of unrestricted deterministic  %
%                           variables                            %
%                       ind: 0-1 integer indicating if standard  %
%                           errors should be calculated or not   %
%                           (0=no,1=yes)                         %
%                       Sigma_pi: Asymptotic covariance          %
%                           matrix of Psi and alpha              %
%                                                                %
%          Output:                                               %
%          	            C: nxn matrix or nx2n matrix             %
%                                                                %
%          This version:        June 1, 2004                     %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2004          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [C,Sigma_c,dC_b,Sigma_b] = cmat(Psi,alpha,beta,epsilon,Z1,Z2,p,d,ind,Sigma_pi)

if nargin<10;
   Sigma_pi = [];
end;

%
% Preliminaries
%

[n T] = size(epsilon);
if isempty(alpha)==1;
   r = 0;
   Z3 = Z2;
   alpha_orth = eye(n);
   beta_orth = eye(n);
else;
   r = size(alpha,2);
   Z3 = [Z2;beta'*Z1];
   alpha_orth = null(alpha');
   beta_orth = null(beta(1:n,:)');
end;

Gamma = eye(n);
for i=1:p-1;
   Gammai =Psi(:,d+1+n*(i-1):d+n*i);
   Gamma = Gamma-Gammai;
end;

%
% Computing C
%

C = (beta_orth/(alpha_orth'*Gamma*beta_orth))*alpha_orth';

%
% Computing standard errors
%

if ind==1;
   nr = size(Z2,1);
   Omega = (1/T)*epsilon*epsilon';
   if r>0;
      xi = ((alpha'*alpha)\alpha')*(Gamma*C-eye(n));
      for i=1:p-1;
         xi = [C;xi];
      end;
      selRow = [zeros(r,nr) eye(r)];
      if p>=2;
         selRow = [[zeros(n*(p-1),d) eye(n*(p-1)) zeros(n*(p-1),nr+r-d-(n*(p-1)))];selRow];
      end;
   else;
      xi = C;
      for i=2:p-1;
         xi = [C;xi];
      end;
      selRow = [zeros(n*(p-1),d) eye(n*(p-1)) zeros(n*(p-1),nr-d-(n*(p-1)))];
   end;
   if isempty(Sigma_pi)==1;
      inv_zz = eye(nr+r)/(Z3*Z3');
      Sigma_C = selRow*inv_zz*selRow';
      Sigma_pi = kron(Sigma_C,Omega);
      Sigma_C = kron(xi'*Sigma_C*xi,C*Omega*C');
   else;
      fix_C = kron(xi'*selRow,C);
      Sigma_C = fix_C*Sigma_pi*fix_C';
      fix_pi = kron(selRow,eye(n));
      Sigma_pi = fix_pi*Sigma_pi*fix_pi';
   end;
   if nargout>=2;
      Sigma_c = Sigma_C;
      if nargout==4;
         dC_b = kron(xi',C);
         Sigma_b = Sigma_pi;
      end;
   else;
      CStd = abs(sqrt(diag(Sigma_C)));
      CStd = vecinv(CStd,n);
      C = [C CStd];
   end;
else;
   if nargout>=2;
      Sigma_c = [];
      if nargout==4;
         dC_b = [];
         Sigma_b = [];
      end;
   end;
end;

%
% end of cmat.m
%
