%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          Computes permanent and transitory components using    %
%          a Beveridge-Nelson type decomposition                 %
%                                                                %
%          Function: permcomp                                    %
%                                                                %
%          Input:                                                %
%                       Psi: nxnq matrix                         %
%                       alpha: nxr matrix                        %
%                       beta: n+q1+dcointxr matrix with          %
%                           cointegration vectors                %
%                       epsilon: nxT matrix with residuals       %
%                       x0: Tpxn matrix                          %
%                       x1: Tpxd matrix                          %
%                       x1_r: Tpxdr matrix                       %
%                       x2: Tpxq matrix                          %
%                       d: number of unrestricted deterministic  %
%                           variables                            %
%                       dcoint: 0-1 variable indicating if a     %
%                           constant or linear trend appears     %
%                           in the cointegration relations       %
%                       p: number of lags for endogenous         %
%                           variables                            %
%                       pe: number of lags for exogenous         %
%                           variables                            %
%                       q1: number of exogenous I(1) variables   %
%                       q0: number of exogenous I(0) variables   %
%                                                                %
%          Output:                                               %
%          	            series: 2xT matrix with permanent and    %
%                           transitory components of x0          %
%                                                                %
%          This version:        January 20, 2005                 %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2005          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [y_t,perm_y,trans_y] = permcomp(Psi,alpha,beta,epsilon,x0,x1,x1_r,x2,d,dcoint,p,pe,q1,q0)

%
% Preliminaries
%

if isempty(x2)==1;
   [w0,w1,w2,w1t]=vecmdata(x0,p,x1);
else;
   [w0,w1,w2,w1t]=vecmdata(x0,p,x1,x2,q1,pe);
end;

if isempty(x1_r)==0;
   w1 = [w1;x1_r'];
end;

[norig Torig] = size(x0);
if norig>Torig;
   [Torig norig] = size(x0);
end;
[n T] = size(epsilon);

if isempty(alpha);
   r = 0;
   beta_y = [];
else;
   r = size(alpha,2);
   beta_y = beta(1:n,:);
end;

if r<n;
   C = cmat(Psi,alpha,beta,epsilon,w1,w2,p,d,0);
   %
   % setting up coefficients on deterministic variables
   % and moving deterministic variables in cointegration
   % relations to the set of deterministic variables
   %
   if d>0;
      delta = Psi(:,1:d);
      D_t = x1(max(p,pe)+1:Torig,:)';
   else;
      delta = [];
      D_t = [];
   end;
   if dcoint==1;
      if r>0;
         delta = [delta alpha*beta(n+q1+dcoint,:)'];
         D_t = [D_t;x1_r'];
      else;
         if isempty(D_t)==1;
            D_t = ones(1,T);
         end;
      end;
   end;
   %
   % Endogenous variables
   %
   y_t = x0(max(p,pe)+1:Torig,:)';
   y_0 = x0(max(p,pe),:)';
   %
   % Accumulating deterministic variables
   %
   sD_t = cumsum(D_t,2);
   %
   % Accumulating ignorance
   %
   seps_t = cumsum(epsilon,2);
else;
   %
   % setting up coefficients on deterministic variables
   % and moving deterministic variables in cointegration
   % relations to the set of deterministic variables
   %
   if d>0;
      D_t = x1(max(p,pe)+1:Torig,:)';
   else;
      D_t = [];
   end;
   if dcoint==1;
      if r>0;
         D_t = [D_t;x1_r'];
      else;
         if isempty(D_t)==1;
            D_t = ones(1,T);
         end;
      end;
   end;
   %
   % Endogenous variables
   %
   y_t = x0(max(p,pe)+1:Torig,:)';
   y_0 = x0(max(p,pe),:)';
end;


%
% Exogenous variables
%

if q1+q0>0;
   %
   % Computing the matrix for the VAR(1) form of the VEC
   %
   J = eye(n);
   if p>=2;
      Phi = Psi(:,d+1:d+n*(p-1));
      J = [J zeros(n,n*(p-2)+r)];
   else;
      if r>0;
         J = [J zeros(n,r)];
      end;
   end;
   if r>0;
      if p>=2;
         A = [Phi alpha];
         if p>=3;
            A = [A;[eye((p-2)*n) zeros((p-2)*n,n+r)]];
         end;
         A = [A;[beta_y'*Phi (beta_y'*alpha)+eye(r)]];
      else;
         A = [[zeros(n,n) alpha];[zeros(r,n) (beta_y'*alpha)+eye(r)]];
      end;
   else;
      A = Phi;
      if p>=3;
         A = [A;[eye(n*(p-2)) zeros(n*(p-2),n)]];
      end;
   end;
   %
   % Inverting I-A; size of A is n(p-1)+r or np+r (for p=1)
   %
   if p>1;
      inv_A = eye(n*(p-1)+r)/(eye(n*(p-1)+r)-A);
   else;
      inv_A = eye((n*p)+r)/(eye((n*p)+r)-A);
   end;
   %
   % The exogenous I(1) variables
   %
   if q1>0;
      x1_t = x2(max(p,pe)+1:Torig,1:q1)'-(x2(max(p,pe),1:q1)'*ones(1,T));
      Psi_x1 = Psi(:,d+(n*(p-1))+1:d+(n*(p-1))+q1*pe);
      if r>0;
         beta_x = beta(n+1:n+q1,:);
      end;
      fixMat = eye(q1);
      for i=1:(pe-1);
         fixMat = [fixMat;eye(q1)];
      end;
      s_Psi_x1 = Psi_x1*fixMat;
      %
      % Setting up B_1
      %
      B_1 = s_Psi_x1;
      if pe>=3;
         B_1 = [B_1;zeros(n*(pe-2),q1)];
      end;
      if r>0;
         B_1 = [B_1;(beta_x'+(beta_y'*s_Psi_x1))];
      end;
      D = J*inv_A*B_1;
   end;
   %
   % The exogenous I(0) variables
   %
   if q0>0;
%      x0_t = x2(max(p,pe)+1:Torig,q1+1:q1+q0)';
      x0_t = cumsum(x2(max(p,pe)+1:Torig,q1+1:q1+q0)',2);
      Psi_x0 = Psi(:,d+(n*(p-1))+q1*pe+1:d+(n*(p-1))+q1*pe+q0*(pe+1));
      fixMat = eye(q0);
      for i=1:pe;
         fixMat = [fixMat;eye(q0)];
      end;
      s_Psi_x0 = Psi_x0*fixMat;
      B_2 = s_Psi_x0;
      if pe>=3;
         B_2 = [B_2;zeros(n*(pe-2),q0)];
      end;
      if r>0;
         B_2 = [B_2;(beta_y'*s_Psi_x0)];
      end;
      F = J*inv_A*B_2;
   end;
end;

%
% Computing permanent and transitory components
%

if q1+q0==0;
   if dcoint==1;
      if r>0;
         perm_y = (C*delta*sD_t)+(C*seps_t)+(y_0*ones(1,T));
      else;
         perm_y = (C*seps_t)+(y_0*ones(1,T));
      end;
   else;
      perm_y = (C*delta*sD_t)+(C*seps_t)+(y_0*ones(1,T));
   end;
   trans_y = y_t-perm_y;
else;
   if r<n;
      if dcoint==1;
         if r>0;
            perm_y = (C*delta*sD_t)+(C*seps_t)+(y_0*ones(1,T));
         else;
            perm_y = (C*seps_t)+(y_0*ones(1,T));
         end;
      else;
         perm_y = (C*delta*sD_t)+(C*seps_t)+(y_0*ones(1,T));
      end;
      if q1>0;
         perm_y = perm_y+(D*x1_t);
      end;
      if q0>0;
         perm_y = perm_y+(F*x0_t);
      end;
      trans_y = y_t-perm_y;
   else;
      perm_y = D*x1_t;
      if q0>0;
         perm_y = perm_y+(F*x0_t);
      end;
      trans_y = y_t-perm_y;
   end;
end;

%
% Adjusting the components for the "mean"
%

if isempty(D_t)==0;
   mean_trans = trans_y*D_t'*inverse(D_t*D_t')*D_t;
   trans_y = trans_y-mean_trans;
   perm_y = perm_y+mean_trans;
end;

%
% end of permcomp.m
%
