%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab function for computing standard errors for     %
%          the short run cofficients and either printing the     %
%          results to file or to a text-string                   %
%                                                                %
%          Function: shortrun                                    %
%                                                                %
%          Input:                                                %
%          		          alpha: nxr matrix                        %
%                       Psi: nx(n(p-1)+d+q0p+q1(p-1)) matrix     %
%                       Omega: nxn matrix                        %
%                       beta: (n+q1+dcoint)xr matrix             %
%                       Z1: (n(p-1)+d+q0p+q1(p-1))xT matrix      %
%                       Z2: (n+q1+dcoint)xT matrix               %
%                       p: number of lags                        %
%                       d: number of deterministic variables     %
%                       deterselect: 1x4 0-1 matrix indicating   %
%                            which deterministics model is used  %
%                       q0: number of exogenous I(0) variables   %
%                       q1: number of exogenous I(1) variables   %
%                       evars: names for endogenous variables    %
%                       xvars0: names for exogenous I(0)         %
%                            variables                           %
%                       xvars1: names for exogenous I(1)         %
%                            variables                           %
%                       deter: names for deterministic variables %
%                       restvec: names for restricted            %
%                            deterministic variables             %
%                       cointvalue: 0-1 indicator if reduced     %
%                            rank is allowed or not (1 equals    %
%                            allowed, 0 equals not allowed)      %
%                       outputfile: output filename              %
%                       ind: 0-1 indicator for action. 0 is      %
%                            write to file, and 1 is write to    %
%                            text string                         %
%                       Sigma_pi: asymptotic covariance matrix   %
%                            for Psi and alpha                   %
%          Output:                                               %
%          	            shortstr: text string with results       %
%                                                                %
%          This version:        May 12, 2006                     %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function shortstr = shortrun(alpha,Psi,Omega,beta,Z1,Z2,p,pe,d,deterselect,q0,q1,evars,xvars0,xvars1,deter,restvec,cointvalue,outputfile,ind,Sigma_pi)

if nargin<21;
   Sigma_pi = [];
end;

curr_ini = find_inisettings;

%
% Preliminaries
%

if isempty(alpha)==1;
   n = size(Omega,1);
   r = 0;
   Z3 = Z2;
else;
   if cointvalue==1;
      [n r] = size(alpha);
      Z3 = [Z2;(beta'*Z1)];
   else;
      [n r] = size(alpha);
      alpha = alpha*beta';
      Z3 = [Z2;Z1];
   end;
end;

n2 = size(Z3,1);
T = size(Z1,2);

if isempty(Sigma_pi)==1;
   inv_zz = eye(n2)/(Z3*Z3');
   Sigma_pi = kron(inv_zz,Omega);
end;
n3 = size(Sigma_pi,1);

Dn = duplication(n);
Dnp = (Dn'*Dn)\Dn';
Sigma_omega = (2/T)*Dnp*kron(Omega,Omega)*Dnp';

dcoint = 0;
trend = 0;
if r>0;
   if deterselect(1,2)==1;
      dcoint = 1;
   elseif deterselect(1,4)==1;
      dcoint = 1;
      trend = 1;
   end;
end;

myvars = vecstr(evars);
mv2add = ' ';
if size(myvars,2)<10;
   for i=2:10-size(myvars,2);
      mv2add = [mv2add ' '];
   end;
end;

%
% Matrix with standard errors
%

StdErr = sqrt(abs(diag(Sigma_pi)));
StdErr = vecinv(StdErr,n);
StdOmega = Dn*sqrt(diag(Sigma_omega));
StdOmega = abs(vecinv(StdOmega,n));
%StdOmega_n = vecinv(sqrt(abs(diag(Dn*Sigma_omega*Dn'))),n);

if r>0;
   VarAlpha = Sigma_pi(n3-(n*r)+1:n3,n3-(n*r)+1:n3);
   VarPi = kron(beta(1:n,:),eye(n))*VarAlpha*kron(beta(1:n,:)',eye(n));
   StdPi = sqrt(abs(diag(VarPi)));
   StdPi = vecinv(StdPi,n);
   if cointvalue==1;
      MyPi = alpha*beta(1:n,:)';
   else;
      MyPi = alpha;
   end;
end;

if (q1>0)&(r>0);
   qPi = alpha*beta(n+1:n+q1,:)';
   VarqPi = kron(beta(n+1:n+q1,:),eye(n))*VarAlpha*kron(beta(n+1:n+q1,:)',eye(n));
   StdqPi = sqrt(abs(diag(VarqPi)));
   StdqPi = vecinv(StdqPi,n);
end;

if d>0;
   deltamat = Psi(:,1:d);
   deltaerr = StdErr(:,1:d);
else;
   deltamat = [];
   deltaerr = [];
end;
r1 = r;
if dcoint==1;
   if r>0;
      if cointvalue==1;
         deltamat = [deltamat alpha*beta(n+q1+dcoint,:)'];
         alphaerr = kron(beta(n+q1+dcoint,:),eye(n))*VarAlpha*kron(beta(n+q1+dcoint,:)',eye(n));
         alphaerr = sqrt(diag(alphaerr));
         alphaerr = abs(vecinv(alphaerr,n));
         deltaerr = [deltaerr alphaerr];
      else;
         deltamat = [deltamat alpha(:,n+q1+dcoint)];
         deltaerr = [deltaerr StdErr(:,n2)];
         r1 = r-1;
      end;
   end;
end;

if ind==0;
   shortstr = '';
   fout = fopen(outputfile,'at');
   if curr_ini.texoutputformat>=2;
      fprintf(fout,'\\subsection*{Short run dynamics}\n\n\\header{For $r$=%1.0f}\n\n',r1);
      if d+dcoint>0;
         if (strcmp(deter,'')==0)&(strcmp(restvec,'')==0);
            fprintf(fout,'\\header{Deterministic variables: %s %s}\n\n',txt2ltxmath(deter),txt2ltxmath(restvec));
         else;
            fprintf(fout,'\\header{Deterministic variables: %s%s}\n\n',txt2ltxmath(deter),txt2ltxmath(restvec));
         end;
         myprintstr = ['%0.' num2str(curr_ini.decimals,'%0.0f') 'f'];
         myprintstr2 = ['(%0.' num2str(curr_ini.decimals,'%0.0f') 'f)'];
         myprintstr3 = ['[%0.' num2str(curr_ini.decimals,'%0.0f') 'f]'];
         r_str = 'rr';
         for i=2:(d+dcoint);
            myprintstr = [myprintstr ' & %0.' num2str(curr_ini.decimals,'%0.0f') 'f'];
            myprintstr2 = [myprintstr2 ' & (%0.' num2str(curr_ini.decimals,'%0.0f') 'f)'];
            myprintstr3 = [myprintstr3 ' & [%0.' num2str(curr_ini.decimals,'%0.0f') 'f]'];
            r_str = [r_str 'r'];
         end;
         fprintf(fout,['\\begin{tabular}{' r_str '}\n']);
         for i=1:n;
            fprintf(fout,[txt2ltxmath(strtrim(myvars(i,:)),'\') ' & ' myprintstr ' \\\\ \n'],deltamat(i,:));
            fprintf(fout,[' & ' myprintstr2 ' \\\\ \n'],deltaerr(i,:));
            fprintf(fout,[' & ' myprintstr3 ' \\\\ \n'],inf2nan(deltamat(i,:)./deltaerr(i,:)));
         end;
         fprintf(fout,'\\end{tabular}\n\n');
         if (cointvalue==1)&((dcoint==0)|(d>0));
            fprintf(fout,'\\noindent\nwhere we have treated $\\beta$ as fixed, i.e., the actual asymptotic distribution\nfor the estimated parameters on deterministic variables is not Gaussian unless\n$\\beta$ is known.\n\n');
         end;
         fprintf(fout,'\\halfruler\n\n');
      end;
      if p>1;
         myprintstr = ['%0.' num2str(curr_ini.decimals,'%0.0f') 'f'];
         myprintstr2 = ['(%0.' num2str(curr_ini.decimals,'%0.0f') 'f)'];
         myprintstr3 = ['[%0.' num2str(curr_ini.decimals,'%0.0f') 'f]'];
         r_str = 'rr';
         for i=2:n;
            myprintstr = [myprintstr ' & %0.' num2str(curr_ini.decimals,'%0.0f') 'f'];
            myprintstr2 = [myprintstr2 ' & (%0.' num2str(curr_ini.decimals,'%0.0f') 'f)'];
            myprintstr3 = [myprintstr3 ' & [%0.' num2str(curr_ini.decimals,'%0.0f') 'f]'];
            r_str = [r_str 'r'];
         end;
         fprintf(fout,'\\header{Endogenous variables: %s}\n\n',txt2ltxmath(evars));
         for j=1:p-1;
            fprintf(fout,['\\begin{tabular}{' r_str '}\n']);
            fprintf(fout,'\\multicolumn{%0.0f}{c}{First difference lag %1.0f} \\\\ \n',n+1,j);
            for i=1:n;
               fprintf(fout,[txt2ltxmath(strtrim(myvars(i,:)),'\') ' & ' myprintstr ' \\\\ \n'],Psi(i,d+1+n*(j-1):d+n*j));
               fprintf(fout,[' & ' myprintstr2 ' \\\\ \n'],StdErr(i,d+1+n*(j-1):d+n*j));
               fprintf(fout,[' & ' myprintstr3 ' \\\\ \n'],inf2nan(Psi(i,d+1+n*(j-1):d+n*j)./StdErr(i,d+1+n*(j-1):d+n*j)));
            end;
            fprintf(fout,'\\end{tabular}\n\n\\halfruler\n\n');
         end;
      end;
      np = d+n*(p-1);
      if q1>0;
         myprintstr = ['%0.' num2str(curr_ini.decimals,'%0.0f') 'f'];
         myprintstr2 = ['(%0.' num2str(curr_ini.decimals,'%0.0f') 'f)'];
         myprintstr3 = ['[%0.' num2str(curr_ini.decimals,'%0.0f') 'f]'];
         r_str = 'rr';
         for i=2:q1;
            myprintstr = [myprintstr ' & %0.' num2str(curr_ini.decimals,'%0.0f') 'f'];
            myprintstr2 = [myprintstr2 ' & (%0.' num2str(curr_ini.decimals,'%0.0f') 'f)'];
            myprintstr3 = [myprintstr3 ' & [%0.' num2str(curr_ini.decimals,'%0.0f') 'f]'];
            r_str = [r_str 'r'];
         end;
         fprintf(fout,'\\header{Exogenous I(1) variables: %s}\n\n',txt2ltxmath(xvars1));
         for j=0:pe-1;
            fprintf(fout,['\\begin{tabular}{' r_str '}\n']);
            fprintf(fout,'\\multicolumn{%0.0f}{c}{First difference lag %1.0f} \\\\ \n',q1+1,j);
            for i=1:n;
               fprintf(fout,[txt2ltxmath(strtrim(myvars(i,:)),'\') ' & ' myprintstr ' \\\\ \n'],Psi(i,np+1+q1*j:np+q1*(j+1)));
               fprintf(fout,[' & ' myprintstr2 ' \\\\ \n'],StdErr(i,np+1+q1*j:np+q1*(j+1)));
               fprintf(fout,[' & ' myprintstr3 ' \\\\ \n'],inf2nan(Psi(i,np+1+q1*j:np+q1*(j+1))./StdErr(i,np+1+q1*j:np+q1*(j+1))));
            end;
            fprintf(fout,'\\end{tabular}\n\n');
         end;
         fprintf(fout,'\\halfruler\n\n');
      end;
      np = np+(q1*pe);
      if q0>0;
         myprintstr = ['%0.' num2str(curr_ini.decimals,'%0.0f') 'f'];
         myprintstr2 = ['(%0.' num2str(curr_ini.decimals,'%0.0f') 'f)'];
         myprintstr3 = ['[%0.' num2str(curr_ini.decimals,'%0.0f') 'f]'];
         r_str = 'rr';
         for i=2:q0;
            myprintstr = [myprintstr ' & %0.' num2str(curr_ini.decimals,'%0.0f') 'f'];
            myprintstr2 = [myprintstr2 ' & (%0.' num2str(curr_ini.decimals,'%0.0f') 'f)'];
            myprintstr3 = [myprintstr3 ' & [%0.' num2str(curr_ini.decimals,'%0.0f') 'f]'];
            r_str = [r_str 'r'];
         end;
         fprintf(fout,'\\header{Exogenous I(0) variables: %s}\n\n',txt2ltxmath(xvars0));
         for j=0:pe;
            fprintf(fout,['\\begin{tabular}{' r_str '}\n']);
            fprintf(fout,'\\multicolumn{%0.0f}{c}{Levels lag %1.0f} \\\\ \n',q0+1,j);
            for i=1:n;
               fprintf(fout,[txt2ltxmath(strtrim(myvars(i,:)),'\') ' & ' myprintstr ' \\\\ \n'],Psi(i,np+1+q0*j:np+q0*(j+1)));
               fprintf(fout,[' & ' myprintstr2 ' \\\\ \n'],StdErr(i,np+1+q0*j:np+q0*(j+1)));
               fprintf(fout,[' & ' myprintstr3 ' \\\\ \n'],inf2nan(Psi(i,np+1+q0*j:np+q0*(j+1))./StdErr(i,np+1+q0*j:np+q0*(j+1))));
            end;
            fprintf(fout,'\\end{tabular}\n\n');
         end;
         fprintf(fout,'\\halfruler\n\n');
      end;
      if r>0;
         if cointvalue==0;
            if dcoint==1;
               alpha = alpha(:,1:r-1);
               n2 = n2-1;
               r = r-1;
            end;
         end;
         myprintstr = ['%0.' num2str(curr_ini.decimals,'%0.0f') 'f'];
         myprintstr2 = ['(%0.' num2str(curr_ini.decimals,'%0.0f') 'f)'];
         myprintstr3 = ['[%0.' num2str(curr_ini.decimals,'%0.0f') 'f]'];
         r_str = 'rr';
         for i=2:r;
            myprintstr = [myprintstr ' & %0.' num2str(curr_ini.decimals,'%0.0f') 'f'];
            myprintstr2 = [myprintstr2 ' & (%0.' num2str(curr_ini.decimals,'%0.0f') 'f)'];
            myprintstr3 = [myprintstr3 ' & [%0.' num2str(curr_ini.decimals,'%0.0f') 'f]'];
            r_str = [r_str 'r'];
         end;
         if cointvalue==1;
            fprintf(fout,'\\header{Estimated $\\alpha$ parameters:}\n\n');
         else;
            fprintf(fout,'\\header{Estimated $\\Pi$ parameters:}\n\n');
         end;
         fprintf(fout,['\\begin{tabular}{' r_str '}\n']);
         for i=1:n;
            fprintf(fout,[txt2ltxmath(strtrim(myvars(i,:)),'\') ' & ' myprintstr ' \\\\ \n'],alpha(i,:));
            fprintf(fout,[' & ' myprintstr2 ' \\\\ \n'],StdErr(i,n2-r+1:n2));
            fprintf(fout,[' & ' myprintstr3 ' \\\\ \n'],inf2nan(alpha(i,:)./StdErr(i,n2-r+1:n2)));
         end;
         fprintf(fout,'\\end{tabular}\n\n\\halfruler\n\n');
         if cointvalue==1;
            myprintstr = ['%0.' num2str(curr_ini.decimals,'%0.0f') 'f'];
            myprintstr2 = ['(%0.' num2str(curr_ini.decimals,'%0.0f') 'f)'];
            myprintstr3 = ['[%0.' num2str(curr_ini.decimals,'%0.0f') 'f]'];
            r_str = 'rr';
            for i=2:n;
               myprintstr = [myprintstr ' & %0.' num2str(curr_ini.decimals,'%0.0f') 'f'];
               myprintstr2 = [myprintstr2 ' & (%0.' num2str(curr_ini.decimals,'%0.0f') 'f)'];
               myprintstr3 = [myprintstr3 ' & [%0.' num2str(curr_ini.decimals,'%0.0f') 'f]'];
               r_str = [r_str 'r'];
            end;
            fprintf(fout,'\\header{Estimated $\\Pi=\\alpha\\beta_y''$ parameters:}\n\n');
            fprintf(fout,['\\begin{tabular}{' r_str '}\n']);
            for i=1:n;
               fprintf(fout,[txt2ltxmath(strtrim(myvars(i,:)),'\') ' & ' myprintstr ' \\\\ \n'],MyPi(i,:));
               fprintf(fout,[' & ' myprintstr2 ' \\\\ \n'],StdPi(i,:));
               fprintf(fout,[' & ' myprintstr3 ' \\\\ \n'],inf2nan(MyPi(i,:)./StdPi(i,:)));
            end;
            fprintf(fout,'\\end{tabular}\n\n\\halfruler\n\n');
            if q1>0;
               myprintstr = ['%0.' num2str(curr_ini.decimals,'%0.0f') 'f'];
               myprintstr2 = ['(%0.' num2str(curr_ini.decimals,'%0.0f') 'f)'];
               myprintstr3 = ['[%0.' num2str(curr_ini.decimals,'%0.0f') 'f]'];
               r_str = 'rr';
               for i=2:q1;
                  myprintstr = [myprintstr ' & %0.' num2str(curr_ini.decimals,'%0.0f') 'f'];
                  myprintstr2 = [myprintstr2 ' & (%0.' num2str(curr_ini.decimals,'%0.0f') 'f)'];
                  myprintstr3 = [myprintstr3 ' & [%0.' num2str(curr_ini.decimals,'%0.0f') 'f]'];
                  r_str = [r_str 'r'];
               end;
               fprintf(fout,'\\header{Estimated $\\Pi=\\alpha\\beta_{x_{1}}''$ parameters:}\n\n');
               fprintf(fout,['\\begin{tabular}{' r_str '}\n']);
               fprintf(fout,[' & ' strrep(txt2ltxmath(xvars1),' ',' & ') ' \\\\ \n']);
               for i=1:n;
                  fprintf(fout,[txt2ltxmath(strtrim(myvars(i,:)),'\') ' & ' myprintstr ' \\\\ \n'],qPi(i,:));
                  fprintf(fout,[' & ' myprintstr2 ' \\\\ \n'],StdqPi(i,:));
                  fprintf(fout,[' & ' myprintstr3 ' \\\\ \n'],inf2nan(qPi(i,:)./StdqPi(i,:)));
               end;
               fprintf(fout,'\\end{tabular}\n\n\\halfruler\n\n');
            end;
         end;
      end;
      myprintstr = ['%0.' num2str(2+curr_ini.decimals,'%0.0f') 'f'];
      myprintstr2 = ['(%0.' num2str(2+curr_ini.decimals,'%0.0f') 'f)'];
      myprintstr3 = ['[%0.' num2str(2+curr_ini.decimals,'%0.0f') 'f]'];
      r_str = 'rr';
      for i=2:n;
         myprintstr = [myprintstr ' & %0.' num2str(2+curr_ini.decimals,'%0.0f') 'f'];
         myprintstr2 = [myprintstr2 ' & (%0.' num2str(2+curr_ini.decimals,'%0.0f') 'f)'];
         myprintstr3 = [myprintstr3 ' & [%0.' num2str(2+curr_ini.decimals,'%0.0f') 'f]'];
         r_str = [r_str 'r'];
      end;
      fprintf(fout,'\\header{Estimated $\\Omega$ parameters:}\n\n');
      fprintf(fout,['\\begin{tabular}{' r_str '}\n']);
      for i=1:n;
         fprintf(fout,[txt2ltxmath(strtrim(myvars(i,:)),'\') ' & ' myprintstr ' \\\\ \n'],Omega(i,:));
         fprintf(fout,[' & ' myprintstr2 ' \\\\ \n'],StdOmega(i,:));
         fprintf(fout,[' & ' myprintstr3 ' \\\\ \n'],inf2nan(Omega(i,:)./StdOmega(i,:)));
      end;
      fprintf(fout,'\\end{tabular}\n\n');
   else;
      fprintf(fout,'SHORT RUN DYNAMICS (for rank=%1.0f)\n',r1);
      fprintf(fout,'\n');
      if d+dcoint>0;
         if (strcmp(deter,'')==0)&(strcmp(restvec,'')==0);
            fprintf(fout,'Deterministic variables: %s %s\n',strtrim(deter),restvec);
         else;
            fprintf(fout,'Deterministic variables: %s%s\n',strtrim(deter),restvec);
         end;
         fprintf(fout,'\n');
         fprintf(fout,'Parameter estimates:\n\n');
         myprintstr = ['%' num2str(6+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
         for i=2:(d+dcoint);
            myprintstr = [myprintstr ' %' num2str(6+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
         end;
         for i=1:n;
            fprintf(fout,[myvars(i,:) mv2add myprintstr '\n'],deltamat(i,:));
         end;
         fprintf(fout,'\nEstimated asymptotic standard errors:\n\n');
         for i=1:n;
            fprintf(fout,[myvars(i,:) mv2add myprintstr '\n'],deltaerr(i,:));
         end;
         fprintf(fout,'\nt-ratios:\n\n');
         for i=1:n;
            fprintf(fout,[myvars(i,:) mv2add myprintstr '\n'],inf2nan(deltamat(i,:)./deltaerr(i,:)));
         end;
         fprintf(fout,'\n');
         if (cointvalue==1)&((dcoint==0)|(d>0));
            fprintf(fout,'where we have treated beta as fixed, i.e., the actual asymptotic distribution\nfor the estimated parameters on deterministic variables is not Gaussian unless\nbeta is known.\n\n');
         end;
      end;
      if p>1;
         myprintstr = ['%' num2str(6+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
         for i=2:n;
            myprintstr = [myprintstr ' %' num2str(6+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
         end;
         if d+dcoint>0;
            fprintf(fout,'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n');
            fprintf(fout,'\n');
         end;
         fprintf(fout,'Endogenous variables: %s\n',evars);
         for j=1:p-1;
            fprintf(fout,'\n               First difference lag %1.0f\n\n',j);
            for i=1:n;
               fprintf(fout,[myvars(i,:) mv2add myprintstr '\n'],Psi(i,d+1+n*(j-1):d+n*j));
            end;
            fprintf(fout,'\nEstimated asymptotic standard errors:\n\n');
            for i=1:n;
               fprintf(fout,[myvars(i,:) mv2add myprintstr '\n'],StdErr(i,d+1+n*(j-1):d+n*j));
            end;
            fprintf(fout,'\nt-ratios:\n\n');
            for i=1:n;
               fprintf(fout,[myvars(i,:) mv2add myprintstr '\n'],inf2nan(Psi(i,d+1+n*(j-1):d+n*j)./StdErr(i,d+1+n*(j-1):d+n*j)));
            end;
         end;
         fprintf(fout,'\n');
      end;
      np = d+n*(p-1);
      if q1>0;
         myprintstr = ['%' num2str(6+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
         for i=2:q1;
            myprintstr = [myprintstr ' %' num2str(6+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
         end;
         if d+dcoint+p>1;
            fprintf(fout,'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n');
            fprintf(fout,'\n');
         end;
         fprintf(fout,'Exogenous I(1) variables: %s\n',xvars1);
         for j=0:pe-1;
            fprintf(fout,'\n               First difference lag %1.0f\n\n',j);
            for i=1:n;
               fprintf(fout,[myvars(i,:) mv2add myprintstr '\n'],Psi(i,np+1+q1*j:np+q1*(j+1)));
            end;
            fprintf(fout,'\nEstimated asymptotic standard errors:\n\n');
            for i=1:n;
               fprintf(fout,[myvars(i,:) mv2add myprintstr '\n'],StdErr(i,np+1+q1*j:np+q1*(j+1)));
            end;
            fprintf(fout,'\nt-ratios:\n\n');
            for i=1:n;
               fprintf(fout,[myvars(i,:) mv2add myprintstr '\n'],inf2nan(Psi(i,np+1+q1*j:np+q1*(j+1))./StdErr(i,np+1+q1*j:np+q1*(j+1))));
            end;
         end;
         fprintf(fout,'\n');
      end;
      np = np+(q1*pe);
      if q0>0;
         myprintstr = ['%' num2str(6+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
         for i=2:q0;
            myprintstr = [myprintstr ' %' num2str(6+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
         end;
         if d+dcoint+p+q1>1;
            fprintf(fout,'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n');
            fprintf(fout,'\n');
         end;
         fprintf(fout,'Exogenous I(0) variables: %s\n',xvars0);
         for j=0:pe;
            fprintf(fout,'\n               Levels lag %1.0f\n\n',j);
            for i=1:n;
               fprintf(fout,[myvars(i,:) mv2add myprintstr '\n'],Psi(i,np+1+q0*j:np+q0*(j+1)));
            end;
            fprintf(fout,'\nEstimated asymptotic standard errors:\n\n');
            for i=1:n;
               fprintf(fout,[myvars(i,:) mv2add myprintstr '\n'],StdErr(i,np+1+q0*j:np+q0*(j+1)));
            end;
            fprintf(fout,'\nt-ratios:\n\n');
            for i=1:n;
               fprintf(fout,[myvars(i,:) mv2add myprintstr '\n'],inf2nan(Psi(i,np+1+q0*j:np+q0*(j+1))./StdErr(i,np+1+q0*j:np+q0*(j+1))));
            end;
         end;
         fprintf(fout,'\n');
      end;
      if r>0;
         if cointvalue==0;
            if dcoint==1;
               alpha = alpha(:,1:r-1);
               n2 = n2-1;
               r = r-1;
            end;
         end;
         myprintstr = ['%' num2str(6+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
         for i=2:r;
            myprintstr = [myprintstr ' %' num2str(6+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
         end;
         fprintf(fout,'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n');
         fprintf(fout,'\n');
         if cointvalue==1;
            fprintf(fout,'Estimated alpha parameters:\n\n');
         else;
            fprintf(fout,'Estimated Pi parameters:\n\n');
         end;
         for i=1:n;
            fprintf(fout,[myvars(i,:) mv2add myprintstr '\n'],alpha(i,:));
         end;
         fprintf(fout,'\nEstimated asymptotic standard errors:\n\n');
         for i=1:n;
            fprintf(fout,[myvars(i,:) mv2add myprintstr '\n'],StdErr(i,n2-r+1:n2));
         end;
         fprintf(fout,'\nt-ratios:\n\n');
         for i=1:n;
            fprintf(fout,[myvars(i,:) mv2add myprintstr '\n'],inf2nan(alpha(i,:)./StdErr(i,n2-r+1:n2)));
         end;
         fprintf(fout,'\n');
         if cointvalue==1;
            myprintstr = ['%' num2str(6+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
            for i=2:n;
               myprintstr = [myprintstr ' %' num2str(6+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
            end;
            fprintf(fout,'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n');
            fprintf(fout,'\n');
            fprintf(fout,'Estimated Pi=alpha*beta_(endogenous)'' parameters:\n\n');
            for i=1:n;
               fprintf(fout,[myvars(i,:) mv2add myprintstr '\n'],MyPi(i,:));
            end;
            fprintf(fout,'\nEstimated asymptotic standard errors:\n\n');
            for i=1:n;
               fprintf(fout,[myvars(i,:) mv2add myprintstr '\n'],StdPi(i,:));
            end;
            fprintf(fout,'\nt-ratios:\n\n');
            for i=1:n;
               fprintf(fout,[myvars(i,:) mv2add myprintstr '\n'],inf2nan(MyPi(i,:)./StdPi(i,:)));
            end;
            fprintf(fout,'\n');
            if q1>0;
               myprintstr = ['%' num2str(6+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
               for i=2:q1;
                  myprintstr = [myprintstr ' %' num2str(6+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
               end;
               fprintf(fout,'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n');
               fprintf(fout,'\n');
               fprintf(fout,'Estimated Pi=alpha*beta_(exogenous I(1))'' parameters:\n\n');
               fprintf(fout,['                ' strrep(xvars1,' ','         ') '\n']);
               for i=1:n;
                  fprintf(fout,[myvars(i,:) mv2add myprintstr '\n'],qPi(i,:));
               end;
               fprintf(fout,'\nEstimated asymptotic standard errors:\n\n');
               for i=1:n;
                  fprintf(fout,[myvars(i,:) mv2add myprintstr '\n'],StdqPi(i,:));
               end;
               fprintf(fout,'\nt-ratios:\n\n');
               for i=1:n;
                  fprintf(fout,[myvars(i,:) mv2add myprintstr '\n'],inf2nan(qPi(i,:)./StdqPi(i,:)));
               end;
               fprintf(fout,'\n');
            end;
         end;
      end;
      if d+dcoint+p+q1+q0+r>1;
         fprintf(fout,'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n');
         fprintf(fout,'\n');
      end;
      myprintstr = ['%' num2str(7+curr_ini.decimals,'%0.0f') '.' num2str(2+curr_ini.decimals,'%0.0f') 'f'];
      for i=2:n;
         myprintstr = [myprintstr ' %' num2str(7+curr_ini.decimals,'%0.0f') '.' num2str(2+curr_ini.decimals,'%0.0f') 'f'];
      end;
      fprintf(fout,'Estimated Omega parameters:\n\n');
      for i=1:n;
         fprintf(fout,[myvars(i,:) ' ' myprintstr '\n'],Omega(i,:));
      end;
      fprintf(fout,'\nEstimated asymptotic standard errors:\n\n');
      for i=1:n;
         fprintf(fout,[myvars(i,:) ' ' myprintstr '\n'],StdOmega(i,:));
      end;
      fprintf(fout,'\nt-ratios:\n\n');
      for i=1:n;
         fprintf(fout,[myvars(i,:) ' ' myprintstr '\n'],inf2nan(Omega(i,:)./StdOmega(i,:)));
      end;
   end;
   fclose(fout);
else;
   shortstr = strvcat(' ',['SHORT RUN DYNAMICS (for rank=' num2str(r1) ')']);
   numchar = size(shortstr,2);
   addcorr = '';
   for i=1:numchar;
      addcorr = strcat(addcorr,'=');
   end;
   shortstr = strvcat(shortstr,addcorr,' ');
   if d+dcoint>0;
      if (strcmp(deter,'')==0)&(strcmp(restvec,'')==0);
         shortstr = strvcat(shortstr,['Deterministic variables: ' strtrim(deter) ' ' restvec],' ','Parameter estimates:',' ');
      else;
         shortstr = strvcat(shortstr,['Deterministic variables: ' deter restvec],' ','Parameter estimates:',' ');
      end;
      delta_str = '';
      prt_val = ['%' num2str(7+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
      for i=1:size(deltamat,2);
         delta_str = [delta_str ' ' prt_val];
      end;
      delta_str = strtrim(delta_str);
      for i=1:n;
         shortstr = strvcat(shortstr,sprintf(['%s%s  ' delta_str],myvars(i,:),mv2add,deltamat(i,:)));
      end;
      shortstr = strvcat(shortstr,' ','Estimated asymptotic standard errors:',' ');
      for i=1:n;
         shortstr = strvcat(shortstr,sprintf(['%s%s  ' delta_str],myvars(i,:),mv2add,deltaerr(i,:)));
      end;
      shortstr = strvcat(shortstr,' ','t-ratios:',' ');
      for i=1:n;
         shortstr = strvcat(shortstr,sprintf(['%s%s  ' delta_str],myvars(i,:),mv2add,inf2nan(deltamat(i,:)./deltaerr(i,:))));
      end;
      shortstr = strvcat(shortstr,' ');
      if (cointvalue==1)&((dcoint==0)|(d>0));
         shortstr = strvcat(shortstr,'where we have treated beta as fixed, i.e., the actual','asymptotic distribution for the estimated parameters on','deterministic variables is not Gaussian unless beta is known.',' ');
      end;
   end;
   if p>1;
      if d+dcoint>0;
         shortstr = strvcat(shortstr,'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',' ');
      end;
      shortstr = strvcat(shortstr,['Endogenous variables: ' evars]);
      %
      gamma_str = '';
      prt_val = ['%' num2str(7+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
      for i=1:n;
         gamma_str = [gamma_str ' ' prt_val];
      end;
      gamma_str = strtrim(gamma_str);
      %
      for j=1:p-1;
         shortstr = strvcat(shortstr,' ',['               First difference lag ' num2str(j)],' ');
         for i=1:n;
            shortstr = strvcat(shortstr,sprintf(['%s%s  ' gamma_str],myvars(i,:),mv2add,Psi(i,d+1+n*(j-1):d+n*j)));
         end;
         shortstr = strvcat(shortstr,' ','Estimated asymptotic standard errors:',' ');
         for i=1:n;
            shortstr = strvcat(shortstr,sprintf(['%s%s  ' gamma_str],myvars(i,:),mv2add,StdErr(i,d+1+n*(j-1):d+n*j)));
         end;
         shortstr = strvcat(shortstr,' ','t-ratios:',' ');
         for i=1:n;
            shortstr = strvcat(shortstr,sprintf(['%s%s  ' gamma_str],myvars(i,:),mv2add,inf2nan(Psi(i,d+1+n*(j-1):d+n*j)./StdErr(i,d+1+n*(j-1):d+n*j))));
         end;
      end;
      shortstr = strvcat(shortstr,' ');
   end;
   np = d+n*(p-1);
   if q1>0;
      if d+dcoint+p>1;
         shortstr = strvcat(shortstr,'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',' ');
      end;
      shortstr = strvcat(shortstr,['Exogenous I(1) variables: ' xvars1]);
      %
      psi_str = '';
      prt_val = ['%' num2str(7+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
      for i=1:q1;
         psi_str = [psi_str ' ' prt_val];
      end;
      psi_str = strtrim(psi_str);
      %
      for j=0:pe-1;
         shortstr = strvcat(shortstr,' ',['               First difference lag ' num2str(j)],' ');
         for i=1:n;
            shortstr = strvcat(shortstr,sprintf(['%s%s  ' psi_str],myvars(i,:),mv2add,Psi(i,np+1+q1*j:np+q1*(j+1))));
         end;
         shortstr = strvcat(shortstr,' ','Estimated asymptotic standard errors:',' ');
         for i=1:n;
            shortstr = strvcat(shortstr,sprintf(['%s%s  ' psi_str],myvars(i,:),mv2add,StdErr(i,np+1+q1*j:np+q1*(j+1))));
         end;
         shortstr = strvcat(shortstr,' ','t-ratios:',' ');
         for i=1:n;
            shortstr = strvcat(shortstr,sprintf(['%s%s  ' psi_str],myvars(i,:),mv2add,inf2nan(Psi(i,np+1+q1*j:np+q1*(j+1))./StdErr(i,np+1+q1*j:np+q1*(j+1)))));
         end;
      end;
      shortstr = strvcat(shortstr,' ');
   end;
   np = np+(q1*pe);
   if q0>0;
      if d+dcoint+p+q1>1;
         shortstr = strvcat(shortstr,'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',' ');
      end;
      shortstr = strvcat(shortstr,['Exogenous I(0) variables: ' xvars0]);
      %
      phi_str = '';
      prt_val = ['%' num2str(7+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
      for i=1:q0;
         phi_str = [phi_str ' ' prt_val];
      end;
      phi_str = strtrim(phi_str);
      %
      for j=0:pe;
         shortstr = strvcat(shortstr,' ',['               Levels lag ' num2str(j)],' ');
         for i=1:n;
            shortstr = strvcat(shortstr,sprintf(['%s%s  ' phi_str],myvars(i,:),mv2add,Psi(i,np+1+q0*j:np+q0*(j+1))));
         end;
         shortstr = strvcat(shortstr,' ','Estimated asymptotic standard errors:',' ');
         for i=1:n;
            shortstr = strvcat(shortstr,sprintf(['%s%s  ' phi_str],myvars(i,:),mv2add,StdErr(i,np+1+q0*j:np+q0*(j+1))));
         end;
         shortstr = strvcat(shortstr,' ','t-ratios:',' ');
         for i=1:n;
            shortstr = strvcat(shortstr,sprintf(['%s%s  ' phi_str],myvars(i,:),mv2add,inf2nan(Psi(i,np+1+q0*j:np+q0*(j+1))./StdErr(i,np+1+q0*j:np+q0*(j+1)))));
         end;
      end;
      shortstr = strvcat(shortstr,' ');
   end;
   if r>0;
      if cointvalue==0;
         if dcoint==1;
            alpha = alpha(:,1:r-1);
            n2 = n2-1;
            r = r-1;
         end;
      end;
      shortstr = strvcat(shortstr,'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',' ');
      if cointvalue==1;
         shortstr = strvcat(shortstr,'Estimated alpha parameters:',' ');
      else;
         shortstr = strvcat(shortstr,'Estimated Pi parameters:',' ');
      end;
      %
      alpha_str = '';
      prt_val = ['%' num2str(7+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
      for i=1:r;
         alpha_str = [alpha_str ' ' prt_val];
      end;
      alpha_str = strtrim(alpha_str);
      %
      for i=1:n;
         shortstr = strvcat(shortstr,sprintf(['%s%s  ' alpha_str],myvars(i,:),mv2add,alpha(i,:)));
      end;
      shortstr = strvcat(shortstr,' ','Estimated asymptotic standard errors:',' ');
      for i=1:n;
         shortstr = strvcat(shortstr,sprintf(['%s%s  ' alpha_str],myvars(i,:),mv2add,StdErr(i,n2-r+1:n2)));
      end;
      shortstr = strvcat(shortstr,' ','t-ratios:',' ');
      for i=1:n;
         shortstr = strvcat(shortstr,sprintf(['%s%s  ' alpha_str],myvars(i,:),mv2add,inf2nan(alpha(i,:)./StdErr(i,n2-r+1:n2))));
      end;
      shortstr = strvcat(shortstr,' ');
      if cointvalue==1;
         shortstr = strvcat(shortstr,'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',' ');
         shortstr = strvcat(shortstr,'Estimated Pi=alpha*beta_(endogenous)'' parameters:',' ');
         %
         pi_str = '';
         prt_val = ['%' num2str(7+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
         for i=1:size(MyPi,2);
            pi_str = [pi_str ' ' prt_val];
         end;
         pi_str = strtrim(pi_str);
         %
         for i=1:n;
            shortstr = strvcat(shortstr,sprintf(['%s%s  ' pi_str],myvars(i,:),mv2add,MyPi(i,:)));
         end;
         shortstr = strvcat(shortstr,' ','Estimated asymptotic standard errors:',' ');
         for i=1:n;
            shortstr = strvcat(shortstr,sprintf(['%s%s  ' pi_str],myvars(i,:),mv2add,StdPi(i,:)));
         end;
         shortstr = strvcat(shortstr,' ','t-ratios:',' ');
         for i=1:n;
            shortstr = strvcat(shortstr,sprintf(['%s%s  ' pi_str],myvars(i,:),mv2add,inf2nan(MyPi(i,:)./StdPi(i,:))));
         end;
         shortstr = strvcat(shortstr,' ');
         if q1>0;
            shortstr = strvcat(shortstr,'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',' ');
            shortstr = strvcat(shortstr,'Estimated Pi_q=alpha*beta_(exogenous I(1))'' parameters:',' ');
            shortstr = strvcat(shortstr,['                 ' strrep(xvars1,' ','        ')]);
            %
            psi_str = '';
            prt_val = ['%' num2str(7+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
            for i=1:q1;
               psi_str = [psi_str ' ' prt_val];
            end;
            psi_str = strtrim(psi_str);
            %
            for i=1:n;
               shortstr = strvcat(shortstr,sprintf(['%s%s  ' psi_str],myvars(i,:),mv2add,qPi(i,:)));
            end;
            shortstr = strvcat(shortstr,' ','Estimated asymptotic standard errors:',' ');
            for i=1:n;
               shortstr = strvcat(shortstr,sprintf(['%s%s  ' psi_str],myvars(i,:),mv2add,StdqPi(i,:)));
            end;
            shortstr = strvcat(shortstr,' ','t-ratios:',' ');
            for i=1:n;
               shortstr = strvcat(shortstr,sprintf(['%s%s  ' psi_str],myvars(i,:),mv2add,inf2nan(qPi(i,:)./StdqPi(i,:))));
            end;
            shortstr = strvcat(shortstr,' ');
         end;
      end;
   end;
   if d+dcoint+p+q1+q0+r>1;
      shortstr = strvcat(shortstr,'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',' ');
   end;
   shortstr = strvcat(shortstr,'Estimated Omega parameters:',' ');
   %
   omega_str = '';
   prt_val = ['%' num2str(7+curr_ini.decimals,'%0.0f') '.' num2str(2+curr_ini.decimals,'%0.0f') 'f'];
   for i=1:n;
      omega_str = [omega_str ' ' prt_val];
   end;
   omega_str = strtrim(omega_str);
   %
   for i=1:n;
      shortstr = strvcat(shortstr,sprintf(['%s%s  ' omega_str],myvars(i,:),mv2add,Omega(i,:)));
   end;
   shortstr = strvcat(shortstr,' ','Estimated asymptotic standard errors:',' ');
   for i=1:n;
      shortstr = strvcat(shortstr,sprintf(['%s%s  ' omega_str],myvars(i,:),mv2add,StdOmega(i,:)));
   end;
   shortstr = strvcat(shortstr,' ','t-ratios:',' ');
   for i=1:n;
      shortstr = strvcat(shortstr,sprintf(['%s%s  ' omega_str],myvars(i,:),mv2add,inf2nan(Omega(i,:)./StdOmega(i,:))));
   end;
   shortstr = strvcat(shortstr,' ');
end;

%
% end of shortrun.m
%
