%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                              %
%  Function: tabbed_dialog                                                     %
%                                                                              %
%  Purpose: Create a tabbed dialog                                             %
%                                                                              %
%  Input:                                                                      %
%           aStruct:     a structure of handles (may be empty) to which        %
%                        handles of objects of the tabbed dialog will          %
%                        be added                                              %
%           cu_ini:      a structure with ini settings (may be empty)          %
%           pos:         position of tabbed dialog (1x4 vector)                %
%           num_tabs:    number of tabs                                        %
%           input_var:   a structure with the following fields                 %
%           fldnames:    a cell array with field names for the tabs            %
%           tags:        a cell array with tags names for the tabs             %
%           strings:     a cell array with text for the tabs                   %
%           callbackstr: a cell array with callbacks for the tabs              %
%           visible:     a cell array with on or off                           %
%           btn_width:   1x2 vector with shown and hidden tab widths           %
%                        (optional)                                            %
%                                                                              %
%  Output:                                                                     %
%        aStruct:    a structure containing all handles of the created         %
%                    objects+any handles passed to the function                %
%                                                                              %
%  This version:     April 4, 2006                                             %
%  Written by:       Anders Warne                                              %
%                    Copyright (c) 2001-2006                                   %
%                                                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function aStruct = tabbed_dialog(aStruct,cu_ini,pos,num_tabs,input_var,btn_width)

fig = get(0,'CurrentFigure');

fixed = 0;
if matlabnumber>=6.0;
   fixed = 1;
end;

darkvec = [0 0 0];
shadevec = [0.5 0.5 0.5];
lightvec = [1 1 1];

if isempty(cu_ini)==0;
   if isfield(cu_ini,'gui_light_color')==1;
      lightvec = cu_ini.gui_light_color;
   end;
   if isfield(cu_ini,'gui_shade_color')==1;
      shadevec = cu_ini.gui_shade_color;
   end;
   if isfield(cu_ini,'gui_dark_color')==1;
      darkvec = cu_ini.gui_dark_color;
   end;
end;

% the bottom x axis

axes('Units','pixels', ...
   'TickLength',[0 0], ...
   'CameraUpVector',[0 1 0], ...
   'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[pos(1) pos(2) pos(3) 1], ...
   'XTickLabelMode','manual', ...
   'YTickLabelMode','manual', ...
   'Tag','gui', ...
   'XColor',darkvec, ...
   'YColor',darkvec, ...
   'ZColor',darkvec);

% shading the bottom x axis up

axes('Units','pixels', ...
   'TickLength',[0 0], ...
   'CameraUpVector',[0 1 0], ...
   'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[pos(1)+1 pos(2)+1 pos(3)-1 1], ...
   'XTickLabelMode','manual', ...
   'YTickLabelMode','manual', ...
   'Tag','gui', ...
   'XColor',shadevec, ...
   'YColor',shadevec, ...
   'ZColor',shadevec);

% the left y axis

axes('Units','pixels', ...
   'TickLength',[0 0], ...
   'CameraUpVector',[0 1 0], ...
   'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[pos(1) pos(2)+1 1 pos(4)], ...
   'XTickLabelMode','manual', ...
   'YTickLabelMode','manual', ...
   'Tag','gui', ...
   'XColor',lightvec, ...
   'YColor',lightvec, ...
   'ZColor',lightvec);

% the top x axis

axes('Units','pixels', ...
   'TickLength',[0 0], ...
   'CameraUpVector',[0 1 0], ...
   'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[pos(1)+1 pos(2)+pos(4)+fixed pos(3)-1 1], ...
   'XTickLabelMode','manual', ...
   'YTickLabelMode','manual', ...
   'Tag','gui', ...
   'XColor',lightvec, ...
   'YColor',lightvec, ...
   'ZColor',lightvec);

% shading the right y axis left

axes('Units','pixels', ...
   'TickLength',[0 0], ...
   'CameraUpVector',[0 1 0], ...
   'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[pos(1)+pos(3)-1 pos(2)+1 1 pos(4)-1], ...
   'XTickLabelMode','manual', ...
   'YTickLabelMode','manual', ...
   'Tag','gui', ...
   'XColor',shadevec, ...
   'YColor',shadevec, ...
   'ZColor',shadevec);

% the right y axis

axes('Units','pixels', ...
   'TickLength',[0 0], ...
   'CameraUpVector',[0 1 0], ...
   'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[pos(1)+pos(3) pos(2) 1 pos(4)+1], ...
   'XTickLabelMode','manual', ...
   'YTickLabelMode','manual', ...
   'Tag','gui', ...
   'XColor',darkvec, ...
   'YColor',darkvec, ...
   'ZColor',darkvec);

%
% removing a few pixels
%

if matlabnumber>=6.0;
   axes('Units','pixels', ...
      'TickLength',[0 0], ...
      'CameraUpVector',[0 1 0], ...
      'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
      'Position',[pos(1)+pos(3)+1 pos(2) 2 2], ...
      'XTickLabelMode','manual', ...
      'YTickLabelMode','manual', ...
      'Tag','gui', ...
      'XColor',get(0,'defaultuicontrolbackgroundcolor'), ...
      'YColor',get(0,'defaultuicontrolbackgroundcolor'), ...
      'ZColor',get(0,'defaultuicontrolbackgroundcolor'));
end;

if nargin<6;
   btn_width = (pos(3)-1)/5;
   btn_1_width = btn_width+1;
   btn_2_width = btn_width-1;
else;
   btn_1_width = btn_width(1);
   btn_2_width = btn_width(2);
end;


%
% create the first tab
%

h_btn = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'ForegroundColor',[0 0 0], ...
         'Position',[pos(1) pos(2)+pos(4)-2 btn_1_width 25], ...
         'String',input_var.strings{1}, ...
         'FontSize',cu_ini.gui_fontsize, ...
         'FontName',cu_ini.gui_fontname, ...
         'FontWeight',cu_ini.gui_fontweight, ...
         'FontAngle',cu_ini.gui_fontangle, ...
         'Visible',input_var.visible{1}, ...
         'Enable','on', ...
         'UserData',1, ...
         'Callback',input_var.callbackstr{1}, ...
         'Tag',input_var.tags{1});

temp_btns = h_btn;

if (isempty(aStruct)==1)|(isstruct(aStruct)==0);
   aStruct = struct(input_var.fldnames{1},h_btn);
else;
   aStruct = setfield(aStruct,input_var.fldnames{1},h_btn);
end;

aStruct.frame1tab1 = uicontrol('Units','pixels', ...
   'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
   'ForegroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[pos(1) pos(2)+pos(4)-10 btn_1_width 11], ...
   'Style','frame', ...
   'Tag','frame1tab1');

aStruct.frame2tab1 = uicontrol('Units','pixels', ...
   'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
   'ForegroundColor',lightvec, ...
   'Position',[pos(1) pos(2)+pos(4)-10 1 11], ...
   'Style','frame', ...
   'Tag','frame2tab1');

aStruct.frame3tab1 = uicontrol('Units','pixels', ...
   'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
   'ForegroundColor',lightvec, ...
   'Position',[pos(1) pos(2)+pos(4) btn_1_width 1], ...
   'Style','frame', ...
   'Visible','off', ...
   'Tag','frame3tab1');

old_curr_row = 1;
add_height = 0;
for i=2:num_tabs;
   if i<=5;
      sub_val = 0;
      if i==2;
         sub_val = 1;
         btn_val = 0;
      else;
         btn_val = btn_val+75.4;
      end;

      h_btn = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'ForegroundColor',[0 0 0], ...
         'Position',[pos(1)+btn_1_width+btn_val pos(2)+pos(4)-2 btn_2_width 23], ...
         'String',input_var.strings{i}, ...
         'FontSize',cu_ini.gui_fontsize, ...
         'FontName',cu_ini.gui_fontname, ...
         'FontWeight',cu_ini.gui_fontweight, ...
         'FontAngle',cu_ini.gui_fontangle, ...
         'Visible',input_var.visible{i}, ...
         'Enable','on', ...
         'Callback',input_var.callbackstr{i}, ...
         'Tag',input_var.tags{i});
       %
       temp_btns = [temp_btns h_btn];
       %
       aStruct = setfield(aStruct,input_var.fldnames{i},h_btn);
       %
       % add frames
       %

      h_f1 = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'ForegroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[pos(1)+btn_1_width+btn_val pos(2)+pos(4)-10 btn_2_width 11], ...
         'Style','frame', ...
         'Tag',['frame1tab' num2str(i)]);

      h_f2 = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'ForegroundColor',lightvec, ...
         'Position',[pos(1)+btn_1_width+btn_val-sub_val pos(2)+pos(4) btn_2_width 1], ...
         'Style','frame', ...
         'Visible','on', ...
         'Tag',['frame3tab' num2str(i)]);

      aStruct = setfield(aStruct,['frame1tab' num2str(i)],h_f1);
      aStruct = setfield(aStruct,['frame3tab' num2str(i)],h_f2);

   else;
      if i==6;
         kids = get(fig,'Children');
         newkids = [];
      end;
      curr_row = ceil(i/5);
      if curr_row>old_curr_row;
         old_curr_row = curr_row;
         btn_val = 0;
         add_height = add_height+20;
      else;
         btn_val = btn_val+75.4;
      end;
      h_btn = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'ForegroundColor',[0 0 0], ...
         'Position',[pos(1)+1+btn_val pos(2)+pos(4)-2+add_height btn_2_width 23], ...
         'String',input_var.strings{i}, ...
         'FontSize',cu_ini.gui_fontsize, ...
         'FontName',cu_ini.gui_fontname, ...
         'FontWeight',cu_ini.gui_fontweight, ...
         'FontAngle',cu_ini.gui_fontangle, ...
         'Visible',input_var.visible{i}, ...
         'Enable','on', ...
         'Callback',input_var.callbackstr{i}, ...
         'Tag',input_var.tags{i});
       %
       temp_btns = [temp_btns h_btn];
       newkids = [newkids;h_btn];
       aStruct = setfield(aStruct,input_var.fldnames{i},h_btn);
   end;
end;

%
% Make sure that any upper row tabs lie underneath a lower row tab
%

if num_tabs>=6;
   set(fig,'Children',[kids;newkids]);
end;

if (iswinxp==1)&(matlabnumber>=7)&(isempty(temp_btns)==0);
   %
   % change the CData property of all pushbuttons
   %
   curr_ini = find_inisettings;
   for i=1:size(temp_btns,2);
      uiPos = get(temp_btns(1,i),'Position');
      sizBack = round(uiPos([4 3]));
      uiColor = get(0,'defaultuicontrolbackgroundcolor');
      cdat = shiftdim(repmat(uiColor,[sizBack(2) 1 sizBack(1)]),2);
      %
      height = size(cdat,1);
      width = size(cdat,2);
      %
      % dark lower edge
      %
      cdat(height,:,1) = curr_ini.gui_dark_color(1,1);
      cdat(height,:,2) = curr_ini.gui_dark_color(1,2);
      cdat(height,:,3) = curr_ini.gui_dark_color(1,3);
      %
      % dark right edge
      %
      cdat(:,width,1) = curr_ini.gui_dark_color(1,1);
      cdat(:,width,2) = curr_ini.gui_dark_color(1,2);
      cdat(:,width,3) = curr_ini.gui_dark_color(1,3);
      %
      % light upper edge
      %
      cdat(1,1:width-1,1) = curr_ini.gui_light_color(1,1);
      cdat(1,1:width-1,2) = curr_ini.gui_light_color(1,2);
      cdat(1,1:width-1,3) = curr_ini.gui_light_color(1,3);
      %
      % light left edge
      %
      cdat(1:height-1,1,1) = curr_ini.gui_light_color(1,1);
      cdat(1:height-1,1,2) = curr_ini.gui_light_color(1,2);
      cdat(1:height-1,1,3) = curr_ini.gui_light_color(1,3);
      %
      % shade lower edge
      %
      cdat(height-1,2:width-1,1) = curr_ini.gui_shade_color(1,1);
      cdat(height-1,2:width-1,2) = curr_ini.gui_shade_color(1,2);
      cdat(height-1,2:width-1,3) = curr_ini.gui_shade_color(1,3);
      %
      % shade right edge
      %
      cdat(2:height-1,width-1,1) = curr_ini.gui_shade_color(1,1);
      cdat(2:height-1,width-1,2) = curr_ini.gui_shade_color(1,2);
      cdat(2:height-1,width-1,3) = curr_ini.gui_shade_color(1,3);
      %
      set(temp_btns(1,i),'CData',cdat);
   end;
end;

%
% end of tabbed_dialog.m
%
