%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          Computes multivariate Wald tests for p versus         %
%          p-k lags (requires p>=2)                              %
%                                                                %
%          Function: waldlagtest                                 %
%                                                                %
%          Input:                                                %
%                       Psi: nxnq matrix                         %
%                       alpha: nxr matrix                        %
%                       beta: n+q1+dcointxr matrix with          %
%                           cointegration vectors                %
%                       p: number of lags under the null         %
%                       pe: number of lags on exogenous          %
%                          variables under the null              %
%                       k: number of fewer lags under            %
%                           the alternative                      %
%                       epsilon: nxT matrix with residuals       %
%                       Z1: nqxT matrix                          %
%                       Z2: nrxT matrix                          %
%                       q1: number of exogenous I(1) variables   %
%                       q0: number of exogenous I(0) variables   %
%                       d: number of unrestricted deterministic  %
%                           variables                            %
%                       t: degrees of freedom from original      %
%                           estimates                            %
%                       Sigma_pi: asymptotic covariance matrix   %
%                            for Psi and alpha                   %
%                                                                %
%          Output:                                               %
%          	            results: matrix with results             %
%                       df: matrix with degrees of freedom       %
%                       status: indicator for success (0=yes     %
%                            1=no)                               %
%                                                                %
%          This version:        January 20, 2005                 %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2005          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [results,df,status] = waldlagtest(Psi,alpha,beta,p,pe,k,epsilon,Z1,Z2,q1,q0,d,t,Sigma_pi)

if nargin<14;
   Sigma_pi = [];
end;

status = 0;
curr_ini = find_inisettings;

%
% Preliminaries
%

if isempty(alpha)==0;
   r = size(alpha,2);
   w = [Z2;beta'*Z1];
else;
   r = 0;
   w = Z2;
end;
nreg = size(w,1);

[n T] = size(epsilon);
Omega = (1/T)*epsilon*epsilon';
theta = vec([Psi alpha]);
if isempty(Sigma_pi)==1;
   inv_ww = eye(nreg)/(w*w');
   Sigma_pi = kron(inv_ww,Omega);
end;

%
% Setting up the matrix with restrictions
%

R2 = [];
R3 = [];
R4 = [];
if p-k-1>0;
   R1 = [zeros(n*k,d) zeros(n*k,n*(p-k-1)) eye(n*k)];
else;
   R1 = [zeros(n*k,d) eye(n*k)];
end;
if r>0;
   R4 = zeros(n*k,r);
end;
if pe-k>0;
   if (q1>0)&(q0==0);
      R1 = [R1 zeros(n*k,q1*pe)];
      R2 = [zeros(q1*k,d) zeros(q1*k,n*(p-1)) zeros(q1*k,q1*(pe-k)) eye(q1*k)];
      if r>0;
         R4 = [R4;zeros(q1*k,r)];
      end;
   elseif (q1>0)&(q0>0);
      R1 = [R1 zeros(n*k,q1*pe) zeros(n*k,q0*(pe+1))];
      R2 = [zeros(q1*k,d) zeros(q1*k,n*(p-1)) zeros(q1*k,q1*(pe-k)) eye(q1*k) zeros(q1*k,q0*(pe+1))];
      R3 = [zeros(q0*k,d) zeros(q0*k,n*(p-1)) zeros(q0*k,q1*pe) zeros(q0*k,q0*(pe-k+1)) eye(q0*k)];
      if r>0;
         R4 = [R4;zeros((q1+q0)*k,r)];
      end;
   elseif (q1==0)&(q0>0);
      R1 = [R1 zeros(n*k,q0*(pe+1))];
      R3 = [zeros(q0*k,d) zeros(q0*k,n*(p-1)) zeros(q0*k,q0*(pe-k+1)) eye(q0*k)];
      if r>0;
         R4 = [R4;zeros(q0*k,r)];
      end;
   end;
else;
   if q1+q0>0;
      R1 = [R1 zeros(n*k,(q1*pe)+(q0*(pe+1)))];
   end;
end;

R = [R1;R2;R3];
R = [R R4];
R = kron(R,eye(n));

if size(R,2)<size(Sigma_pi,1);
   s1 = size(Sigma_pi,1)-size(R,2)+1;
   s2 = size(Sigma_pi,2);
   Sigma_pi = Sigma_pi(s1:s2,s1:s2);
end;
corr_mat = R*Sigma_pi*R';

if size(corr_mat,1)==rank(corr_mat,eps);
   W = (R*theta)'*(corr_mat\(R*theta));
else;
   status = 1;
   results = [];
   df = [];
   return;
end;

if (curr_ini.bartserial==1)&(isalmostequal(q1+q0,0)==1);
   k0 = p-k;
   k1 = p;
   bc = 1+((1/(2*T*k)))*( (n+1)*k + n*((k1*k1)-(k0*k0)) )+(1/(T*k*n*n))*( n*(floor(k1/2)-floor(k0/2)) -(n/2)*( k1*(k1-1) -k0*(k0-1)));
   W = W/bc;
end;
if curr_ini.sscorrect==0;
   df = size(R,1);
   pval = chis_cdf(W,df);
   results = [W 1-pval];
else;
   W = (t/(T*size(R,1)))*W;
   df = [size(R,1) t];
   pval = fdis_cdf(W,df(1,1),df(1,2));
   results = [W 1-pval];
end;

%
% end of waldlagtest.m
%
