%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab function for estimating nxn matrix B_0 such    %
%          that Omega = B_0*B_0' and all other parameters (if    %
%          required) simultaneously. Provides a FIML estimator   %
%          of all requested parameters                           %
%                                                                %
%          Function: warne                                       %
%                                                                %
%          Input:                                                %
%                       tm: structure with model info            %
%                       type_var: string determining type of     %
%                           restrictions to use                  %
%                       keep_rest: restrictions that can be      %
%                           put on B_0 alone                     %
%                       del_rest: restrictions that cannot be    %
%                           put on B_0 alone                     %
%                       use_psi_rest: restrictions on C*B_0      %
%                           which are linearly independent of    %
%                           restrictions placed on [Psi alpha]   %
%                           when they are linked to [Psi alpha]  %
%                       Psi: parameters on short-run dynamics    %
%                           and deterministics                   %
%                       alpha: parameters on levels relations    %
%                       beta: cointegration space                %
%                       epsilon: residuals                       %
%                       Z0: endogenous variables in changes      %
%                       Z1: levels variables                     %
%                       Z2: changes and deterministics           %
%                       B_init: initial values for B_0           %
%                       step: parameter to multiply with         %
%                           parameter change in algorithm        %
%                       waitbar: 0-1 indicator whether to        %
%                           display a progress dialog or not     %
%                       use_case: case to estimate (if 0 then    %
%                           all cases are checked)               %
%                                                                %
%          Output:                                               %
%                        status: indicator of success            %
%                           (0 OK, 1 failure, 2 bad initial      %
%                            B matrix)                           %
%                        abort: string indicating if estimation  %
%                           was aborted or not                   %
%          	             B_0: nxn matrix with structural         %
%                           coefficients                         %
%                        Psi_n: nxk matrix with parameters on    %
%                           deterministic variables, lagged      %
%                           first differences, and exogenous     %
%                           I(0) variables                       %
%                        alpha_n: nxr matrix with weights on the %
%                           cointegration relations              %
%                        beta_n: (n+q1+dcoint)xr matrix with     %
%                           cointegration space parameters       %
%                        C_n: nxn matrix with reduced form       %
%                           parameters on the trends             %
%                        A: nxn matrix with structural long-run  %
%                           coefficients on the trends           %
%                        stdbeta: estimated asymptotic           %
%                           conditional standard errors for beta %
%                        S: n^2xq matrix mapping free parameters %
%                           gamma into vec(B_0)                  %
%                        Sigma_theta: covariance matrix for      %
%                           Psi and alpha                        %
%                        Sigma_B0: covariance matrix for B_0     %
%                        Sigma_B0_adj: adjusted covariance       %
%                        matrix for B_0                          %
%                        Sigma_A: covariance matrix for A        %
%                        Sigma_full: Complete covariance matrix  %
%                           of all the model parameters (except  %
%                           for beta)                            %
%                        infomat: 1x2 vector with number of      %
%                           iterations and value of convergence  %
%                           criterion                            %
%                        Rmat: the restrictions matrix           %
%                        epsilon_new: nxT matrix with reduced    %
%                           form residuals                       %
%                        num_rest: the actual number of          %
%                           restrictions implied by the          %
%                           identifying assumptions              %
%                        dC_theta: the derivative of C with      %
%                           respect to vec([Psi alpha])          %
%                        max_case: case with the maximum         %
%                           value for the loglikelihood function %
%                                                                %
%          This version:        June 16, 2006                    %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [status,abort,B_0,Psi_n,alpha_n,beta_n,C_n,A,stdbeta,S,Sigma_theta,Sigma_B0,Sigma_B0_adj,Sigma_A,infomat,Rmat,epsilon_new,num_rest,dC_theta,dB_theta,max_case,rank_test] = warne(tm,type_var,keep_rest,del_rest,use_psi_rest,Psi,alpha,beta,epsilon,Z0,Z1,Z2,B_init,step,waitbar,use_case,d_coint);

if nargin<17;
   d_coint = 0;
end;
if nargin<16;
   use_case = 0;
end;
if nargin<15;
   waitbar = 0;
end;
if nargin<14;
   step = 1;
end;

r = size(alpha,2);
[n T] = size(epsilon);
Omega = (1/T)*epsilon*epsilon';
max_case = [];

local_status = 0;
status = 1;
abort = '';
B_0 = [];
A = [];
S = [];
Sigma_B0 = [];
Sigma_B0_adj = [];
Sigma_A = [];
Sigma_full = [];
Sigma_theta = [];
infomat = [];
R_l = [];
Rmat = [];
stdbeta = [];
epsilon_new = epsilon;
num_rest = [];
dC_theta = [];
dB_theta = [];
Psi_n = [];
alpha_n = [];
beta_n = [];
C_n = [];
num_rest = [];
rank_test = [];

%
% check the ini file
%

curr_ini = find_inisettings;

if curr_ini.convcrit<10;
   ch_convcrit = ['1.0e-00' num2str(curr_ini.convcrit)];
else;
   ch_convcrit = ['1.0e-0' num2str(curr_ini.convcrit)];
end;
ccrit = mystr2num(ch_convcrit);

%
% initialize B if it is not already given
%

if isempty(B_init)==1;
   [U,V] = eig(Omega);
   B_init = U*sqrt(V);
else;
   %
   % check to make sure B_init is non-singular
   %
   if rank(B_init)~=n;
      [U,V] = eig(Omega);
      B_init = U*sqrt(V);
      local_status = 2;
   end;
end;

%
% retrieving restrictions placed on alpha
%

if tm.restrict(1,1)==1;
   file = strcat(tm.modelsdir,'G-',int2str(n),'x',int2str(r),'-',tm.modelfile,'.rest');
   G = loaddata(file);
   [Rmat_alpha,rvec_alpha] = rmatrix(G,n,r);
   if isempty(tm.Z2)==0;
      Rmat_alpha = [zeros(size(G,1),size(tm.Z2,1)*n) Rmat_alpha];
   end;
else;
   Rmat_alpha = [];
   rvec_alpha = [];
end;

%
% we need to rescale alpha and beta to make sure the cointegration relations
% are roughly on the same scale as the first difference variables. ones the
% algorithm has finished we need to rescale alpha and the estimated asymptotic
% covariance matrix
%

if (r>0)&(tm.restrict(1,2)==0);
   beta_fix = beta(1:r,:);
   beta = beta/beta_fix;
   alpha = alpha*beta_fix';
end;

%
% retrieve restrictions on beta (only possible when r>0 and r<n+q1+dcoint
%

if (curr_ini.reestbeta==1)&(tm.restrict(1,2)==1);
   sivec = zeros(r,1);
   %
   % bH_std is needed to check the gradient of the likelihood function
   % wrt the beta parameters
   %
   for i=1:r;
      file = strcat(tm.modelsdir,'H_',num2str(i),'-',num2str(n),'x',num2str(r),'x',num2str(tm.dcoint+tm.q1),'-',tm.modelfile,'.rest');
      if fileexist(file)==1;
         h = loaddata(file);
         if d_coint>0;
            h = [h zeros(size(h,1),d_coint);zeros(d_coint,size(h,2)) eye(d_coint)];
         end;
         if i==1;
            H_std = h;
            bH_std = h;
         else;
            H_std = [H_std h];
            bH_std = [[bH_std zeros(size(bH_std,1),size(h,2))];[zeros(size(h,1),size(bH_std,2)) h]];
         end;
         sivec(i,1) = size(h,2);
      end;
   end;
   %
   % Get the normalizing variables
   %
   file = strcat(tm.modelsdir,'normalize-',num2str(n),'x',num2str(r),'x',num2str(tm.dcoint+tm.q1),'-',tm.modelfile,'.norm');
   if fileexist(file)==1;
      normvec = loaddata(file);
   else;
      normvec = [];
   end;
   %
   % Setting up the normalizing restrictions
   %
   if isempty(normvec)==0;
      nq1 = size(beta,1);
      addrestmat = zeros(nq1*r,r);
      for i=1:r;
         thisnorm = normvec(i,1);
         addrestmat(thisnorm+nq1*(i-1),i) = 1;
      end;
   else;
      addrestmat = [];
   end;
   %
   % constructing restriction matrices for beta
   %
   Knr = commutation(n+tm.q1+tm.dcoint+d_coint,r);
   bH_ort = null(bH_std','r');
   bH_ort = [bH_ort addrestmat];
   bRmat = bH_ort'*Knr';
   if isempty(addrestmat)==1;
      brvec = zeros(size(bRmat,1),1);
   else;
      brvec = [zeros(size(bRmat,1)-r,1);ones(r,1)];
   end;
   bH_std = null(bH_ort','r');
   %
   % actual beta restrictions
   %
   I_mat = eye(size(beta,1));
   h = [];
   H = [];
   tot_free = 0;
   %
   % get restrictions on beta
   %
   for i=1:r;
      file = strcat(tm.modelsdir,'H_',num2str(i),'-',num2str(n),'x',num2str(r),'x',num2str(tm.dcoint+tm.q1),'-',tm.modelfile,'.rest');
      if fileexist(file)==1;
         H_i = loaddata(file);
         if d_coint>0;
            H_i = [H_i zeros(size(H_i,1),d_coint);zeros(d_coint,size(H_i,2)) eye(d_coint)];
         end;
         %
         % determine which rows in H_i are linearly independend
         %
         H_fix = H_i(normvec(i),:);
         j = 1;
         while j<=size(H_i,1);
           if size(H_fix,1)<size(H_fix,2);
              if j~=normvec(i);
                 H_test = [H_fix;H_i(j,:)];
                 if rank(H_test)==size(H_test,1);
                    H_fix = H_test;
                 end;
              end;
              j = j+1;
           else;
              j = size(H_i,1)+1;
           end;
         end;
         H_i = H_i/H_fix;
         h_i = H_i(:,1);
         h = [h;h_i];
         if size(H_i,2)>1;
            H_i = H_i(:,2:size(H_i,2));
         else;
            H_i = [];
         end;
         H_struc.elem(i).H_i = H_i;
         H_struc.elem(i).tot_free = tot_free;
         tot_free = tot_free+size(H_i,2);
      end;
   end;
   %
   % now we construct H from the above info
   %
   for i=1:r;
      if isempty(H_struc.elem(i).H_i)==1;
         H = [H;zeros(size(beta,1),tot_free)];
      else;
         H_i = H_struc.elem(i).H_i;
         my_free = H_struc.elem(i).tot_free;
         if my_free>0;
            H_first = zeros(size(beta,1),my_free);
         else;
            H_first = [];
         end;
         if tot_free-(my_free+size(H_i,2))>0;
            H_last = zeros(size(beta,1),tot_free-(my_free+size(H_i,2)));
         else;
            H_last = [];
         end;
         H = [H;[H_first H_i H_last]];
      end;
   end;
end;

%
% computing some matrices we'll need later on but don't need
% to recompute
%
M_01 = (1/T)*Z0*Z1';
M_11 = (1/T)*Z1*Z1';
M_10 = M_01';
if isempty(Z2)==0;
   M_21 = (1/T)*Z2*Z1';
   M_12 = M_21';
   M_22 = (1/T)*Z2*Z2';
else;
   M_21 = [];
   M_12 = [];
   M_22 = [];
end;

%
% initializing restrictions and certain parameters
%

if strcmp(type_var,'short_run')==1;
   file = strcat(tm.modelsdir,'F-',int2str(n),'x',int2str(r),'-',tm.modelfile,'.rest');
   F = loaddata(file);
   [Rmat_sr,rvec_sr] = rmatrix(F,n,n);
   R_l = [];
   Rmat_lr = [];
   rvec_lr = [];
   if isempty(Rmat_alpha)==1;
      if isempty([Psi alpha])==0;
         Rmat = [zeros(size(Rmat_sr,1),size([Psi alpha],2)*n) Rmat_sr];
      else;
         Rmat = Rmat_sr;
      end;
      rvec = rvec_sr;
      Rmat_psi = [];
      rvec_psi = [];
   else;
      Rmat = [[Rmat_alpha zeros(size(Rmat_alpha,1),n*n)];[zeros(size(Rmat_sr,1),size(Rmat_alpha,2)) Rmat_sr]];
      rvec = [rvec_alpha;rvec_sr];
      Rmat_psi = Rmat_alpha;
      rvec_psi = rvec_alpha;
   end;
   Rmat_B = Rmat_sr;
   rvec_B = rvec_sr;
   if r<n;
      C = cmat(Psi,alpha,beta,epsilon,Z1,Z2,tm.p,tm.d,0);
   else;
      C = cmat_stat(Psi,alpha,beta,epsilon,Z1,Z2,tm.p,tm.d,0,[]);
   end;
   xi = xi_mat(C,Psi,alpha,tm.p,tm.pe,r,tm.d,tm.q1,tm.q0);
elseif strcmp(type_var,'long_run')==1;
   file = strcat(tm.modelsdir,'LF-',int2str(n),'x',int2str(r),'-',tm.modelfile,'.rest');
   LF = loaddata(file);
   [Rmat_lr,rvec_lr] = rmatrix(LF,n,n);
   Rmat_sr = [];
   rvec_sr = [];
   %
   % need to compute a C matrix also for this case, i.e., r==n
   %
   if r<n;
      C = cmat(Psi,alpha,beta,epsilon,Z1,Z2,tm.p,tm.d,0,[]);
   else;
      %
      % now r=n
      %
      C = cmat_stat(Psi,alpha,beta,epsilon,Z1,Z2,tm.p,tm.d,0,[]);
   end;
   xi = xi_mat(C,Psi,alpha,tm.p,tm.pe,r,tm.d,tm.q1,tm.q0);
else;
   file = strcat(tm.modelsdir,'F-',int2str(n),'x',int2str(r),'-',tm.modelfile,'.rest');
   F = loaddata(file);
   [Rmat_sr,rvec_sr] = rmatrix(F,n,n);
   file = strcat(tm.modelsdir,'LF-',int2str(n),'x',int2str(r),'-',tm.modelfile,'.rest');
   LF = loaddata(file);
   [Rmat_lr,rvec_lr] = rmatrix(LF,n,n);
   if r<n;
      C = cmat(Psi,alpha,beta,epsilon,Z1,Z2,tm.p,tm.d,0);
   else;
      C = cmat_stat(Psi,alpha,beta,epsilon,Z1,Z2,tm.p,tm.d,0,[]);
   end;
   xi = xi_mat(C,Psi,alpha,tm.p,tm.pe,r,tm.d,tm.q1,tm.q0);
   %
end;


%
% Check if we have restrictions on C
%
if tm.restrict(1,3)==1;
   file = strcat(tm.modelsdir,'C-',int2str(n),'x',int2str(r),'-',tm.modelfile,'.rest');
   Cres = loaddata(file);
   [Rmat_C,rvec_C] = rmatrix(Cres,n,n);
   g_c = Rmat_C*kron(xi',C);
   if rank(g_c)<size(g_c,1);
      [red_rest,use_rest] = getdelvars(g_c',[]);
      Rmat_C = Rmat_C(use_rest,:);
      rvec_C = rvec_C(use_rest,1);
   end;
else;
   Rmat_C = [];
   rvec_C = [];
end;

Nn = ncommutation(n);

%
% beginning the loop over all cases to examine
%

if use_case==0;
   num_cases = size(keep_rest.entry,1);
else;
   num_cases = 1;
end;

res.max_values = zeros(num_cases,1);

showwaitbar = 0;
if (waitbar==1)&(num_cases>1);
   input_var.title = ['Progress for ' int2str(num_cases) ' cases'];
   input_var.facecolor = curr_ini.progress_facecolor;
   input_var.startfacecolor = curr_ini.progress_startfacecolor;
   input_var.edgecolor = curr_ini.progress_edgecolor;
   input_var.bgcolor = curr_ini.progress_bgcolor;
   input_var.stop = 0;
   input_var.clock = 1;
   input_var.label = 'Case number:';
   input_var.name = 'ML Estimation of B';
   mywait = progressdlg(0,input_var);
%   set(mywait,'Name','ML Estimation of B');
   set(mywait,'Color',get(0,'defaultuicontrolbackgroundcolor'));
   drawnow;
   showwaitbar = 1;
elseif (waitbar==1)&(num_cases==1);
   showwaitbar = 2;
end;

for c_num=1:num_cases;
   if use_case==0;
      case_num = c_num;
   else;
      case_num = use_case;
   end;
   %
   % show a progress dialog
   %
   if showwaitbar==1;
      abort = get(mywait,'UserData');
      if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
         break;
      else;
         progressdlg([c_num/num_cases c_num]);
      end;
   end;
   %
   R_l = [];
   if strcmp(type_var,'long_run')==1;
      if isempty(Rmat_alpha)==1;
         Rmat = [Rmat_lr*kron(B_init'*xi',C) Rmat_lr*kron(eye(n),C)];
         rvec = rvec_lr;
      else;
         Rmat = [[Rmat_alpha zeros(size(Rmat_alpha,1),n*n)];[Rmat_lr*kron(B_init'*xi',C) Rmat_lr*kron(eye(n),C)]];
         rvec = [rvec_alpha;rvec_lr];
      end;
      if isempty(del_rest.entry(case_num,1).rest)==1;
         Rmat_B = Rmat_lr*kron(eye(n),C);
         rvec_B = rvec_lr;
         Rmat_psi = Rmat_alpha;
         rvec_psi = rvec_alpha;
      else;
         Rmat_B = Rmat_lr(keep_rest.entry(case_num,1).rest,:)*kron(eye(n),C);
         rvec_B = rvec_lr(keep_rest.entry(case_num,1).rest,1);
         Rmat_psi = [Rmat_alpha;Rmat_lr(del_rest.entry(case_num,1).rest,:)*kron(B_init'*xi',C)];
         rvec_psi = [rvec_alpha;rvec_lr(del_rest.entry(case_num,1).rest,1)];
      end;
      R_l = Rmat_lr(keep_rest.entry(case_num,1).rest,:);
   elseif strcmp(type_var,'joint')==1;
      %
      if isempty(Rmat_alpha)==1;
         Rmat = [[Rmat_lr*kron(B_init'*xi',C) Rmat_lr*kron(eye(n),C)];[zeros(size(Rmat_sr,1),size([Psi alpha],2)*n) Rmat_sr]];
         rvec = [rvec_lr;rvec_sr];
      else;
         Rmat = [[Rmat_alpha zeros(size(Rmat_alpha,1),n*n)];[Rmat_lr*kron(B_init'*xi',C) Rmat_lr*kron(eye(n),C)];[zeros(size(Rmat_sr,1),size(Rmat_alpha,2)) Rmat_sr]];
         rvec = [rvec_alpha;rvec_lr;rvec_sr];
      end;
      if isempty(del_rest.entry(case_num,1).rest)==1;
         Rmat_B = [Rmat_lr*kron(eye(n),C);Rmat_sr];
         rvec_B = [rvec_lr;rvec_sr];
         Rmat_psi = Rmat_alpha;
         Rvec_psi = rvec_alpha;
      else;
         d_rest = del_rest.entry(case_num,1).rest-size(Rmat_sr,1);
         Rmat_B = [Rmat_sr;Rmat_lr*kron(eye(n),C)];
         Rmat_B = Rmat_B(keep_rest.entry(case_num,1).rest,:);
         rvec_B = [rvec_sr;rvec_lr];
         rvec_B = rvec_B(keep_rest.entry(case_num,1).rest,1);
         Rmat_psi = [Rmat_alpha;Rmat_lr(d_rest,:)*kron(B_init'*xi',C)];
         rvec_psi = [rvec_alpha;rvec_lr(d_rest,1)];
      end;
      R_l = [Rmat_sr;Rmat_lr];
      R_l = R_l(keep_rest.entry(case_num,1).rest,:);
      R_l(1:size(Rmat_sr,1),:) = zeros(size(Rmat_sr));
   end;
   %
   Psi_n = Psi;
   alpha_n = alpha;
   C_n = C;
   xi_n = xi;
   beta_n = beta;
   %
   S = null(Rmat);
   d = (Rmat'/(Rmat*Rmat'))*rvec;
   %
   %
   % From Rmat_B and rvec_B we derive initial value of B which satisfy the restrictions
   % that can be placed on B
   %
   S_B = null(Rmat_B);
   d_B = (Rmat_B'/(Rmat_B*Rmat_B'))*rvec_B;
   gamma = (S_B'*S_B)\(S_B'*(vec(B_init)-d_B));
   B_0 = vecinv(S_B*gamma+d_B,n);
   %
   if isempty(C_n)==0;
      A = C_n*B_0;
   else;
      A = B_0;
   end;
   if rank(B_0)<n;
      if showwaitbar==1;
         set(mywait,'UserData','done');
         close(mywait);
         drawnow;
      end;
      return;
   end;
   %
   %
   % Define the regressors
   %
   Z = [];
   if r>0;
      Z = beta_n'*Z1;
   end;
   Z = [Z2;Z];
   ZZ_inv = (Z*Z')\eye(size(Z,1));
   %
   % Initializing matrices and computing values for restrictions
   % and gradient
   %
   rest_vec = evaluate_rest(Psi_n,alpha_n,B_0,C_n,Rmat_alpha,rvec_alpha,Rmat_lr,rvec_lr,Rmat_sr,rvec_sr,Rmat_C,rvec_C);
   [max_rest,max_ind] = max(abs(rest_vec));
   Omega_n = Omega;
   old_lnl = loglike_fcn(B_0,epsilon_new);
   lnl_diff = abs(abs(old_lnl)-abs(loglike(epsilon)));
   max_val = max([lnl_diff max_rest]);
   rest_diff = max_rest;
   old_max_rest = max_rest;
   old_max_val = 0;
   %
   inside_waitbar = 0;
   if (showwaitbar==2)&(abs(max_val-old_max_val)>ccrit);
      input_var.title = ['Progress for maximum of ' int2str(curr_ini.maxit) ' iterations'];
      input_var.facecolor = curr_ini.progress_facecolor;
      input_var.startfacecolor = curr_ini.progress_startfacecolor;
      input_var.edgecolor = curr_ini.progress_edgecolor;
      input_var.bgcolor = curr_ini.progress_bgcolor;
      input_var.stop = 1;
      input_var.clock = 0;
      input_var.name = 'ML Estimation of B';
      mywait = progressdlg(0,input_var);
%      set(mywait,'Name','ML Estimation of B');
      set(mywait,'Color',get(0,'defaultuicontrolbackgroundcolor'));
      drawnow;
      inside_waitbar = 1;
   end;
   %
   it = 0;
   kill_B = 0;
%   while (it<=curr_ini.maxit)&(abs(max_val-old_max_val)>ccrit);
%   while (it<=curr_ini.maxit)&(abs(max_val)>ccrit);
   while (it<=curr_ini.maxit)&(max([lnl_diff rest_diff])>ccrit);
      it = it+1;
      if inside_waitbar==1;
         abort = get(mywait,'UserData');
         if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
            break;
         else;
            progressdlg([it/curr_ini.maxit max([lnl_diff rest_diff])]);
         end;
      end;
      old_max_val = max_val;
      %
      % NOTE: The order we choose here for conditioning may influence the results
      %       since all the restrictions that can be handled by B will hold when
      %       we estimate Psi and alpha conditional on B. Ideally we should check
      %       all combinations of long-run restrictions. Now such restrictions are
      %       only handled by B or by C, not combined...
      %
      % estimate B_0 conditional on [Psi_n alpha_n]
      %
      B_0_inv = B_0\eye(n);
      grad = -T*S_B'*kron(B_0_inv,B_0_inv')*(vec(B_0')-( kron(Omega_n,eye(n))*vec(B_0_inv) ) );
      max_grad = max(abs(grad));
      it_B = 0;
      while ((it_B<=curr_ini.maxit)&(max_grad>ccrit));
         it_B = it_B+1;
         i_b = kron(eye(n),B_0_inv);
         info_mat = 2*T*S_B'*i_b'*Nn*i_b*S_B;
         %
%         if rank(info_mat)<size(info_mat,1);
%            kill_B = 1;
%            it_B = curr_ini.maxit+1;
%            break;
%         end;
         %
         change = info_mat\grad;
         if max(abs(change))>=1;
            change = step*(1/max(abs(change)))*change;
         else;
            change = step*change;
         end;
         gamma = gamma+change;
         max_nan = max(max(isnan(vecinv(S_B*gamma+d_B,n))));
         if max_nan==1;
            kill_B = 1;
            it_B = curr_ini.maxit+1;
            break;
         end;
         B_0 = vecinv(S_B*gamma+d_B,n);
         if rank(B_0)<n;
            if showwaitbar>0;
               set(mywait,'UserData','done');
               close(mywait);
               drawnow;
            end;
            return;
         end;
         [Rmat_B,rvec_B,S_B,d_B] = rest_B(type_var,keep_rest.entry(case_num,1).rest,del_rest.entry(case_num,1).rest,C_n,Rmat_sr,rvec_sr,Rmat_lr,rvec_lr,n);
         B_0_inv = B_0\eye(n);
         grad = -T*S_B'*kron(B_0_inv,B_0_inv')*(vec(B_0')-( kron(Omega_n,eye(n))*vec(B_0_inv) ) );
         max_grad = max(abs(grad));
      end;
      %
      % if we were successful with the B_0 estimation we will continue
      %
      kill_psi = 0;
      if kill_B==0;
         %
         % in case there are long-run restrictions: estimate [Psi_n alpha_n] conditional on B_0
         %
         if (strcmp(type_var,'short_run')==0)&((num_cases>1)|(size(keep_rest.entry(1,1).rest,1)~=((n*(n-1))/2))|(isempty(del_rest.entry(1,1).rest)==0));
            %
            % store old estimates
            %
            alpha_old = alpha_n;
            Psi_old = Psi_n;
            Omega_old = Omega_n;
            Omega_old_inv = Omega_n\eye(n);
            use_psi_rest_local = use_psi_rest.entry(case_num,1).rest;
            %
            Omega_n = B_0*B_0';
            rest_vec = evaluate_rest(Psi_n,alpha_n,B_0,C_n,Rmat_alpha,rvec_alpha,Rmat_lr,rvec_lr,Rmat_sr,rvec_sr,Rmat_C,rvec_C);
            %
            % remake Z if needed
            %
            if ((tm.restrict(1,2)==1)|(r>0))&(curr_ini.reestbeta==1);
               Z = [];
               if r>0;
                  Z = beta_n'*Z1;
               end;
               Z = [Z2;Z];
               ZZ_inv = (Z*Z')\eye(size(Z,1));
            end;
            ZZ_Omega = kron(ZZ_inv,Omega_n);
            %
            % unrestricted psi_hat doesn't seem to work if we had
            % originally estimated the model under alpha restrictions...
            %
      %      psi_hat = vec(Z0*Z'*ZZ_inv);
            psi_hat = vec([Psi_n alpha_n]);
            %
            % create unrestricted Psi and alpha from psi_hat
            %
            Theta_hat = vecinv(psi_hat,n);
            if isempty(Psi)==0;
               Psi_hat = Theta_hat(:,1:size(Psi,2));
               if isempty(alpha)==0;
                  alpha_hat = Theta_hat(:,size(Psi,2)+1:size(Theta_hat,2));
               else;
                  alpha_hat = [];
               end;
            else;
               alpha_hat = Theta_hat;
               Psi_hat = [];
            end;
            %
            % create unrestricted residuals
            %
            if (isempty(Z2)==0)&(r>0);
              epsilon_hat = Z0-(Psi_hat*Z2)-(alpha_hat*beta_n'*Z1);
            elseif (isempty(Z2)==0)&(r==0);
              epsilon_hat = Z0-(Psi_hat*Z2);
            elseif (isempty(Z2)==1)&(r>0);
              epsilon_hat = Z0-(alpha_hat*beta_n'*Z1);
            else;
              epsilon_hat = Z0;
            end;
            %
            % create unrestricted C from Psi_hat and alpha_hat
            %
            if r<n;
               C_hat = cmat(Psi_hat,alpha_hat,beta_n,epsilon_hat,Z1,Z2,tm.p,tm.d,0);
            else;
               C_hat = cmat_stat(Psi_hat,alpha_hat,beta_n,epsilon_hat,Z1,Z2,tm.p,tm.d,0,[]);
            end;
            %
            % now evaluate the restrictions under the unrestricted Psi and alpha
            %
            rest_vec_hat = evaluate_rest(Psi_hat,alpha_hat,B_0,C_hat,Rmat_alpha,rvec_alpha,Rmat_lr,rvec_lr,Rmat_sr,rvec_sr,Rmat_C,rvec_C);
            max_rest_n = max(abs(rest_vec_hat));
            Psi_n = Psi_hat;
            alpha_n = alpha_hat;
            C_n = C_hat;
            psi_n = psi_hat;
      %      rest_hat_psi_alpha = rest_vec_hat(1:size(Rmat_alpha,1)+size(Rmat_lr,1),1);
            %
            % now we loop through the adjustment until convergence
            %
            it_psi = 0;
            while (it_psi<=curr_ini.maxit)&(max_rest_n>ccrit)&(kill_psi==0);
               it_psi = it_psi+1;
               rest_hat_psi_alpha = rest_vec_hat(1:size(Rmat_alpha,1)+size(Rmat_C,1)+size(Rmat_lr,1),1);
               xi_n = xi_mat(C_n,Psi_n,alpha_n,tm.p,tm.pe,r,tm.d,tm.q1,tm.q0);
               Rmat_psi_alpha = rest_psi_alpha(B_0,C_n,xi_n,Rmat_lr,Rmat_alpha,Rmat_C);
               %
               if rank(Rmat_psi_alpha)<size(Rmat_psi_alpha,1);
                 [red_rest_local,use_psi_rest_local] = getdelvars(Rmat_psi_alpha',[]);
               end;
               %
%               if isempty(use_psi_rest.entry(case_num,1).rest)==0;
               if isempty(use_psi_rest_local)==0;
                  Rmat_psi_alpha = Rmat_psi_alpha(use_psi_rest_local,:);
                  rest_hat_psi_alpha = rest_hat_psi_alpha(use_psi_rest_local,1);
               end;
%               Rmat_psi_alpha_orth = null(Rmat_psi_alpha);
%               c_Rmat_psi_alpha = null(Rmat_psi_alpha_orth');
%               Rmat_psi_alpha = c_Rmat_psi_alpha';
               %
               check_psi = Rmat_psi_alpha*ZZ_Omega*Rmat_psi_alpha';
               %
               if rank(check_psi)==size(check_psi,1);
                  psi_adj = ((ZZ_Omega*Rmat_psi_alpha')/(Rmat_psi_alpha*ZZ_Omega*Rmat_psi_alpha'))*rest_hat_psi_alpha;
               else;
                  psi_adj = 0;
                  kill_psi = 1;
               end;
               %
               % get a new value for psi_n
               %
               psi_n = psi_n-psi_adj;
               %
               % create restricted estimators of Psi and alpha
               %
               Theta_n = vecinv(psi_n,n);
               if isempty(Psi)==0;
                  Psi_n = Theta_n(:,1:size(Psi,2));
                  if isempty(alpha)==0;
                     alpha_n = Theta_n(:,size(Psi,2)+1:size(Theta_n,2));
                  else;
                     alpha_n = [];
                  end;
               else;
                  alpha_n = Theta_n;
                  Psi_n = [];
               end;
               %
               % create restricted residuals
               %
               if (isempty(Z2)==0)&(r>0);
                 epsilon_new = Z0-(Psi_n*Z2)-(alpha_n*beta_n'*Z1);
               elseif (isempty(Z2)==0)&(r==0);
                 epsilon_new = Z0-(Psi_n*Z2);
               elseif (isempty(Z2)==1)&(r>0);
                 epsilon_new = Z0-(alpha_n*beta_n'*Z1);
               else;
                 epsilon_new = Z0;
               end;
               %
               % create unrestricted C from Psi_hat and alpha_hat
               %
               if r<n;
                  C_n = cmat(Psi_n,alpha_n,beta_n,epsilon_new,Z1,Z2,tm.p,tm.d,0);
               else;
                  C_n = cmat_stat(Psi_n,alpha_n,beta_n,epsilon_new,Z1,Z2,tm.p,tm.d,0,[]);
               end;
               %
               % now evaluate the restrictions under the unrestricted Psi and alpha
               %
               rest_vec_hat = evaluate_rest(Psi_n,alpha_n,B_0,C_n,Rmat_alpha,rvec_alpha,Rmat_lr,rvec_lr,Rmat_sr,rvec_sr,Rmat_C,rvec_C);
               max_rest_n = max(abs(rest_vec_hat));
            end;
            %
            if isempty(use_psi_rest_local)==0;
               if size(use_psi_rest_local)==size(use_psi_rest.entry(case_num,1).rest);
                  if sum(abs(use_psi_rest_local-use_psi_rest.entry(case_num,1).rest))>eps;
                     kill_psi = 1;
                  end;
               else;
                  kill_psi = 1;
               end;
            end;
            %
            if kill_psi==1;
               %
               % get the old estimates
               %
               alpha_n = alpha_old;
               Psi_n = Psi_old;
               Omega_n = Omega_old;
               Omega_n_inv = Omega_old_inv;
            end;
            %
            % Now we update the residuals and the covariance matrix of the residuals
            %
            if (isempty(Z2)==0)&(r>0);
              epsilon_new = Z0-(Psi_n*Z2)-(alpha_n*beta_n'*Z1);
            elseif (isempty(Z2)==0)&(r==0);
              epsilon_new = Z0-(Psi_n*Z2);
            elseif (isempty(Z2)==1)&(r>0);
              epsilon_new = Z0-(alpha_n*beta_n'*Z1);
            else;
              epsilon_new = Z0;
            end;
            Omega_n = (1/T)*epsilon_new*epsilon_new';
            %
            % now we should update beta if that's selected
            %
            if (curr_ini.reestbeta==1)&(tm.restrict(1,2)==1)&(kill_psi==0);
               %
               % store old estimates
               %
               %
               Omega_n_inv = Omega_n\eye(n);
               %
               m_1 = H'*kron(alpha_n'*Omega_n_inv*alpha_n,M_11)*H;
               m_2 = H'*kron(alpha_n'*Omega_n_inv,I_mat);
               if isempty(M_12)==0;
                  c_1 = vec((M_10-(M_12*Psi_n')));
               else;
                  c_1 = vec(M_10);
               end;
               c_2 = kron(alpha_n,M_11)*h;
               beta_phi = m_1\(m_2*(c_1-c_2));
               v_beta = h+(H*beta_phi);
               beta_n = vecinv(v_beta,size(beta_n,1));
               %
               % create new C from Psi_n, alpha_n, and beta_n
               %
               C_n = cmat(Psi_n,alpha_n,beta_n,epsilon_new,Z1,Z2,tm.p,tm.d,0);
               %
               % we still need to construct S_11_alpha for conditional standard errors for beta
               %
               if isempty(Z2)==0;
                  newR0 = Z0-((Psi_n+(alpha_n*beta_n'*(M_12/M_22)))*Z2);
                  newR1 = Z1-((M_12/M_22)*Z2);
               else;
                  newR0 = Z0;
                  newR1 = Z1;
               end;
               %
               % Constructing unrestricted beta estimate
               %
               S_00_new = (1/T)*newR0*newR0';
               S_10_new = (1/T)*newR1*newR0';
               S_11_new = (1/T)*newR1*newR1';
               S_01_new = (1/T)*newR0*newR1';
               alpharort = null(alpha_n');
               alpharbar = alpha_n/(alpha_n'*alpha_n);
               S_11_alpha = S_11_new-(S_10_new*alpharort*((alpharort'*S_00_new*alpharort)\(alpharort'*S_01_new)));
               %
               % we're done and now need to update all the parameters that were'nt already updated
               %
               if (isempty(Z2)==0)&(r>0);
                 epsilon_new = Z0-(Psi_n*Z2)-(alpha_n*beta_n'*Z1);
               elseif (isempty(Z2)==0)&(r==0);
                 epsilon_new = Z0-(Psi_n*Z2);
               elseif (isempty(Z2)==1)&(r>0);
                 epsilon_new = Z0-(alpha_n*beta_n'*Z1);
               else;
                 epsilon_new = Z0;
               end;
               Omega_n = (1/T)*epsilon_new*epsilon_new';
            elseif (curr_ini.reestbeta==1)&(r>0)&(kill_psi==0);
               %
               % store old estimates
               %
               Omega_n_inv = inverse(Omega_n);
               %
               if isempty(Z2)==0;
                  beta_n = inverse(M_11)*(M_10-(M_12*Psi_n'))*Omega_n_inv*alpha_n*inverse(alpha_n'*Omega_n_inv*alpha_n);
               else;
                  beta_n = inverse(M_11)*M_10*Omega_n_inv*alpha_n*inverse(alpha_n'*Omega_n_inv*alpha_n);
               end;
               %
               % create new C from Psi_n, alpha_n, and beta_n
               %
               if r<n;
                  C_n = cmat(Psi_n,alpha_n,beta_n,epsilon_new,Z1,Z2,tm.p,tm.d,0);
               else;
                  C_n = cmat_stat(Psi_n,alpha_n,beta_n,epsilon_new,Z1,Z2,tm.p,tm.d,0,[]);
               end;
               %
               % we're done and now need to update all the parameters that were'nt already updated
               %
               if isempty(Z2)==0;
                 epsilon_new = Z0-(Psi_n*Z2)-(alpha_n*beta_n'*Z1);
               else;
                 epsilon_new = Z0-(alpha_n*beta_n'*Z1);
               end;
               Omega_n = (1/T)*epsilon_new*epsilon_new';
            end;
         end;
         %
         % now we evaluate the restrictions and the gradient
         %
         rest_vec = evaluate_rest(Psi_n,alpha_n,B_0,C_n,Rmat_alpha,rvec_alpha,Rmat_lr,rvec_lr,Rmat_sr,rvec_sr,Rmat_C,rvec_C);
         [max_rest,max_ind] = max(abs(rest_vec));
         if strcmp(type_var,'short_run')==0;
            xi_n = xi_mat(C_n,Psi_n,alpha_n,tm.p,tm.pe,r,tm.d,tm.q1,tm.q0);
         end;
         %
         new_lnl = loglike_fcn(B_0,epsilon_new);
         max_val = max([new_lnl-old_lnl max_rest]);
         lnl_diff = abs(abs(new_lnl)-abs(old_lnl));
         rest_diff = abs(max_rest-old_max_rest);
         old_max_rest = max_rest;
         %
         old_lnl = new_lnl;
         %
         % update the value for the restrictions on B
         %
         [Rmat_B,rvec_B,S_B,d_B] = rest_B(type_var,keep_rest.entry(case_num,1).rest,del_rest.entry(case_num,1).rest,C_n,Rmat_sr,rvec_sr,Rmat_lr,rvec_lr,n);
      else;
         it = curr_ini.maxit+1;
      end;
   end;
   %
   % store all values in a structure
   %
   if (max_rest<=ccrit)&(it<=curr_ini.maxit);
      res.max_values(c_num,1) = new_lnl;
      res.case(c_num,1).B_0 = B_0;
      res.case(c_num,1).Psi = Psi_n;
      res.case(c_num,1).alpha = alpha_n;
      res.case(c_num,1).beta = beta_n;
      res.case(c_num,1).Omega = Omega_n;
      res.case(c_num,1).epsilon_new = epsilon_new;
      res.case(c_num,1).infomat = [it lnl_diff];
      res.case(c_num,1).max_rest = max_rest;
      res.case(c_num,1).S_B = S_B;
      res.case(c_num,1).gamma = gamma;
      res.case(c_num,1).Rmat_B = Rmat_B;
      res.case(c_num,1).B_0_inv = B_0_inv;
      res.case(c_num,1).C = C_n;
      res.case(c_num,1).xi = xi_n;
      res.case(c_num,1).R_l = R_l;
      if (strcmp(type_var,'short_run')==0)&(curr_ini.reestbeta==1)&(tm.restrict(1,2)==1);
         res.case(c_num,1).S_11_alpha = S_11_alpha;
         res.case(c_num,1).Omega_n_inv = Omega_n_inv;
      end;
   end;
end;

%
% now we need to determine which of these cases has the highest value
% for the log-likelihood function. That case is selected and its
% parameters are then used below
%

[m_v,max_case] = max(abs(res.max_values));

if m_v<1.0e-016;
   if showwaitbar>0;
      set(mywait,'UserData','done');
      close(mywait);
      drawnow;
   end;
   return;
end;

B_0 = res.case(max_case,1).B_0;
Psi_n = res.case(max_case,1).Psi;
alpha_n = res.case(max_case,1).alpha;
beta_n = res.case(max_case,1).beta;
Omega_n = res.case(max_case,1).Omega;
epsilon_new = res.case(max_case,1).epsilon_new;
infomat = res.case(max_case,1).infomat;
S_B = res.case(max_case,1).S_B;
gamma = res.case(max_case,1).gamma;
Rmat_B = res.case(max_case,1).Rmat_B;
B_0_inv = res.case(max_case,1).B_0_inv;
C_n = res.case(max_case,1).C;
xi_n = res.case(max_case,1).xi;
R_l = res.case(max_case,1).R_l;
max_rest = res.case(max_case,1).max_rest;
if (strcmp(type_var,'short_run')==0)&(curr_ini.reestbeta==1)&(tm.restrict(1,2)==1);
   S_11_alpha = res.case(max_case,1).S_11_alpha;
   Omega_n_inv = res.case(max_case,1).Omega_n_inv;
end;

%
% close the progress dialog
%

if showwaitbar>0;
   set(mywait,'UserData','done');
   close(mywait);
   drawnow;
end;

%
% in case beta has been reestimated then we need to recompute its conditional standard errors
%

if (strcmp(type_var,'short_run')==0)&(curr_ini.reestbeta==1)&(tm.restrict(1,2)==1);
   %
   % recompute ths conditional standard errors for beta
   %
   stdbeta = zeros(size(beta_n));
   for i=1:r;
      %
      % Selecting the restrictions from H_std
      %
      si = sivec(i,1);
      if si>=2;
         sj = size(H_std,2);
         if i==1;
            Hsel = [eye(si);zeros(sj-si,si)];
         elseif i==r;
            Hsel = [zeros(sj-si,si);eye(si)];
         else;
            sk = sivec(1,1);
            for j=2:i-1;
               sk = sk+sivec(j,1);
            end;
            Hsel = [zeros(sk,si);eye(si);zeros(sj-si-sk,si)];
         end;
         Hi = H_std*Hsel;
         %
         % Determining the normalizations
         %
         normelem = normvec(i,1);
         myexit = 0;
         for j=1:si;
            %
            % Fix Hi to make sure that the normalizing row
            % contains one unit element and zeros
            %
            normmatrix = Hi(normelem,:);
            for k=1:(n+tm.q1+tm.dcoint+d_coint);
               if k~=normelem;
                  checkHi = [normmatrix;Hi(k,:)];
                  if rank(checkHi*checkHi')==size(checkHi,1);
                     normmatrix = checkHi;
                     if size(normmatrix,1)==size(Hi,2);
                        break;
                     end;
                  end;
               end;
            end;
            Hi = Hi/normmatrix;
            check = Hi(normelem,j);
            if (abs(check)>0)&(check~=1);
               Hi(:,j) = (1/Hi(normelem,j))*Hi(:,j);
               check = Hi(normelem,j);
            end;          
            if check==1;
               %
               % Split Hi into hi and H_i
               %
               if j==1;
                  hi = Hi(:,1);
                  H_i = Hi(:,2:si);
               elseif j==si;
                  hi = Hi(:,si);
                  H_i = Hi(:,1:si-1);
               else;
                  hi = Hi(:,j);
                  H_i = [Hi(:,1:j-1) Hi(:,j+1:si)];
               end;
               %
               % Check H_i to make sure it is of full rank
               %
               check2 = H_i'*H_i;
               if abs(det(check2))>=.000000001;
                  myexit=1;
               end;
            end;
            if myexit==1;
               break;
            end;
         end;
         %
         % Computing the asymptotic covariance for the
         % si-1 free parameters of column i in betar
         %
         information = alpha_n(:,i)'*Omega_n_inv*alpha_n(:,i)*H_i'*S_11_alpha*H_i;
         covbeta = (1/T)*(H_i/information)*H_i';
         %
         % Computing the asyptotic standard errors and
         % including them in stdbeta
         %
         stdbeta(:,i) = sqrt(abs(diag(covbeta)));
      end;
   end;
else;
   stdbeta = [];
end;

%
% transform parameters back to the scale they're meant for
%

if (r>0)&(tm.restrict(1,2)==0);
   beta_n = beta_n*beta_fix;
   alpha_n = alpha_n/beta_fix';
   if r<n;
      C_n = cmat(Psi_n,alpha_n,beta_n,epsilon_new,Z1,Z2,tm.p,tm.d,0);
   else;
      C_n = cmat_stat(Psi_n,alpha_n,beta_n,epsilon_new,Z1,Z2,tm.p,tm.d,0,[]);
   end;
   xi_n = xi_mat(C_n,Psi_n,alpha_n,tm.p,tm.pe,r,tm.d,tm.q1,tm.q0);
end;

%
% check if we should fix the signs of the shocks
%

H_sign = change_ir_signs(tm.signshock,B_0,n);

B_0 = B_0*H_sign;
B_0_inv = B_0\eye(n);
gamma = ((S_B'*S_B)\S_B')*kron(H_sign',eye(n))*S_B*gamma;

%
% done with estimation. compute estimate of the asymptotic covariance matrix
%
% the lines below are only here for testing purposes
%
[Rmat,rvec,S,d] = create_rest(type_var,Psi_n,alpha_n,B_0,C_n,xi_n,Rmat_sr,rvec_sr,Rmat_lr,rvec_lr,Rmat_alpha,rvec_alpha,Rmat_C,rvec_C,n);
%i_phi = info_phi(B_0,B_0*B_0',Z1,Z2,beta_n,S,r,0);
%Sigma_full = S*(i_phi\S');

%
% Try a theoretically more sound method
%

if strcmp(type_var,'short_run')==0;
   %
   % linearized restrictions
   %
   if isempty(del_rest.entry(max_case,1).rest)==0;
      Rmat_psi_alpha = rest_psi_alpha(B_0,C_n,xi_n,Rmat_lr,Rmat_alpha,Rmat_C);
      if rank(Rmat_psi_alpha)<size(Rmat_psi_alpha,1);
         [red_rest_local,use_psi_rest_local] = getdelvars(Rmat_psi_alpha',[]);
         Rmat_psi_alpha = Rmat_psi_alpha(use_psi_rest_local,:);
      end;
      if rank(Rmat_psi_alpha*Rmat_psi_alpha')<size(Rmat_psi_alpha,1);
         %
         % need to remove all rows of Rmat_psi_alpha for which
         % the row rank is unaffected when the row is removed
         %
         t_ones = ones(size(Rmat_psi_alpha,1),1);
         row_keep = (1:1:size(Rmat_psi_alpha,1))';
         for i=1:size(Rmat_psi_alpha,1);
            t_ones(i,1) = 0;
            rows_keep = row_keep(t_ones==1);
            new_rank = rank(Rmat_psi_alpha(rows_keep,:)*Rmat_psi_alpha(rows_keep,:)');
            old_rank = rank(Rmat_psi_alpha*Rmat_psi_alpha');
            if new_rank<old_rank;
               t_ones(i,1) = 1;
            end;
         end;
         rows_keep = row_keep(t_ones==1);
         Rmat_psi_alpha = Rmat_psi_alpha(rows_keep,:);
      end;
   end;
   %
   % update the data just in case
   %
   Z = [];
   if r>0;
      Z = beta_n'*Z1;
   end;
   Z = [Z2;Z];
   ZZ_inv = (Z*Z')\eye(size(Z,1));
   ZZ_Omega = kron(ZZ_inv,Omega_n);
   if isempty(del_rest.entry(max_case,1).rest)==0;
      part1_adj = Rmat_psi_alpha*ZZ_Omega;
      part2_adj = part1_adj*Rmat_psi_alpha';
      Sig_theta_adj = part1_adj'*(part2_adj\part1_adj);
      Sigma_theta = ZZ_Omega-Sig_theta_adj;
   else;
      Sigma_theta = tm.Sigma_theta;
   end;
   %
   % Need to determine dvec(B_0)/dvec([Psi alpha])'.
   % We shall do so in steps.
   %
   dC_theta = kron(xi_n',C_n);
   %
   % computing dvec(R)/dvec(C)'
   %
   dR_C = dr_c(R_l,n);
   %
   % computing dvec(S)/dvec(R)'
   %
   dS_R = ds_r(S_B,Rmat_B);
   %
   % computing dvec(Omega)/dgamma'
   %
   dO_g = do_g(B_0,S_B);
   %
   % computing dvec(Omega)/vec(R)'
   %
   dO_R = do_r(B_0,Rmat_B);
   %
   % computing dgamma/dvec(R)'
   %
   dg_R = dg_r(dO_R,dO_g);
   %
   % computing dvec(B)/dvec(R)'
   %
   dB_R = db_r(dS_R,dg_R,gamma,S_B,n);
   %
   % computing dvec(B)/dvec(theta)'
   %
   dB_theta = dB_R*dR_C*dC_theta;
   %
   % Now we have the tools to compute the covariance matrix of B_0
   %
   i_b = kron(eye(n),B_0_inv);
   info_B0 = 2*S_B'*i_b'*Nn*i_b*S_B;
   Sigma_B0 = (1/T)*(S_B/info_B0)*S_B';
   %
   % Finally, we wish to compute the covariance matrix for B_0,[Psi alpha]
   %
   Sigma_cov = dB_theta*Sigma_theta;
   Sigma_full = [[Sigma_theta Sigma_cov'];[Sigma_cov (dB_theta*Sigma_theta*dB_theta')+Sigma_B0]];
else;
   Rmat_psi_alpha = Rmat_alpha;
   xi_n = xi_mat(C_n,Psi_n,alpha_n,tm.p,tm.pe,r,tm.d,tm.q1,tm.q0);
   if isempty(C_n)==0;
      dC_theta = kron(xi_n',C_n);
   else;
      dC_theta = [];
   end;
   dB_theta = 0;
   i_b = kron(eye(n),B_0_inv);
   Sigma_theta = tm.Sigma_theta;
   info_B0 = 2*S_B'*i_b'*Nn*i_b*S_B;
   Sigma_B0 = (1/T)*(S_B/info_B0)*S_B';
   Sigma_cov = zeros(n*n,size(Sigma_theta,2));
   Sigma_full = [[Sigma_theta Sigma_cov'];[Sigma_cov Sigma_B0]];
end;

%num_rest = size(Sigma_full,1)-rank(Sigma_full,eps)-(size(Rmat_psi_alpha,1)-size(del_rest,1));
num_rest = size(Rmat_lr,1)+size(Rmat_sr,1);
%num_rest = size(S,1)-size(S,2)-size(Rmat_C,1)-size(Rmat_alpha,1);

%
% Alternative covariance matrix
%
%i_phi = info_phi(B_0,B_0*B_0',Z1,Z2,beta_n,S,r,0);
%Sigma_full = S*(i_phi\S');
%Sigma_theta = Sigma_full(1:size(Sigma_full,1)-(n*n),1:size(Sigma_full,1)-(n*n));
%Sigma_B0 = Sigma_full(size(Sigma_full,1)-(n*n)+1:size(Sigma_full,1),size(Sigma_full,1)-(n*n)+1:size(Sigma_full,1));

%
% determine the asymptotic covariance matrix for vec(B_0) and vec([Psi alpha])
%

Sigma_B0_adj = Sigma_full(size(Sigma_full,1)-(n*n)+1:size(Sigma_full,1),size(Sigma_full,1)-(n*n)+1:size(Sigma_full,1));

if isempty(C_n)==0;
   A = C_n*B_0;
   b_i = kron(B_0',eye(n));
   i_c = kron(eye(n),C_n);
   if strcmp(type_var,'short_run')==0;
      dA_theta = (b_i*dC_theta)+(i_c*dB_theta);
   else;
      dA_theta = (b_i*dC_theta);
   end;
   dA_B = i_c;
else;
   A = B_0;
end;

%
% Computing the covariance matrix for A
%
if isempty(C_n)==0;
   if size(dA_theta,2)<size(Sigma_theta,1);
      dA_theta = [zeros(size(dA_theta,1),size(Sigma_theta,1)-size(dA_theta,2)) dA_theta];
   end;
   Sigma_A = (dA_theta*Sigma_theta*dA_theta')+(dA_B*Sigma_B0*dA_B');
else;
   Sigma_A = Sigma_B0;
end;

%
% test for local idenitification
%

test_mat = [T*Sigma_B0;Rmat_B];
if rank(test_mat,eps)==floor(n*n);
   status = 0;
end;
rank_test = rank(test_mat,eps);

if (local_status==2)&(status==0);
   status = 2;
end;

%
% end of warne.m
%
