function [m,out1,out2]=wk1read_fcn(filename, r, c, rng)
%WK1READ Read spreadsheet (WK1) file.
%   A = WK1READ('FILENAME') reads all the numeric data from a Lotus WK1
%   spreadsheet file named FILENAME into matrix A. Cells containing text
%   are set to zero.
%
%   A = WK1READ('FILENAME',R,C) reads data from a Lotus WK1 spreadsheet
%   file starting at row R and column C, into the matrix A.  R and C
%   are zero-based so that R=C=0 is the first cell of the spreadsheet.
%
%   A = WK1READ('FILENAME',R,C,RNG)  specifies a cell range or named
%   range for selecting data from the spreadsheet.  A cell range is
%   specified by RNG = [R1 C1 R2 C2] where (R1,C1) is the upper-left
%   corner of the data to be read and (R2,C2) is the lower-right 
%   corner. RNG can also be specified using spreadsheet notation as 
%   in RNG = 'A1..B7' or a named range like 'Sales'.
%
%   [A,B] = WK1READ(...) same as above with text data stored in cell
%   array B.
%
%   See also WK1WRITE, CSVREAD, CSVWRITE.

%   Brian M. Bourgault 10/22/93
%   Copyright 1984-2001 The MathWorks, Inc. 
%
%   $Revision: 5.18 $  $Date: 2001/04/15 12:02:20 $
%
% include WK1 constants
%
%wk1const;
if 0
LOTTABLE    = [ 24 24 ];
LOTQRANGE   = [ 25 25 ];
LOTPRANGE   = [ 26 8 ];
LOTSRANGE   = [ 27 8 ];
LOTFRANGE   = [ 28 8 ];
LOTKRANGE1  = [ 29 9 ];
LOTDRANGE   = [ 32 0 ];
LOTKRANGE2  = [ 35 9 ];
LOTPROTECT  = [ 36 1 ];
LOTFOOTER   = [ 37 242 ];
LOTHEADER   = [ 38 242 ];
LOTSETUP    = [ 39 40 ];
LOTTITLES   = [ 42 16 ];
LOTGRAPH    = [ 45 437 ];
LOTNGRAPH   = [ 46 455 ];
LOTFORMAT   = [ 48 1 ];
LOTWINDOW   = [ 50 144 ];
LOTSTRING   = [ 51 0 ];
LOTLOCKPASS = [ 55 0 ];
LOTLOCKED   = [ 56 0 ];
LOTQSET     = [ 60 0 ];
LOTCQUERY   = [ 61 0 ];
LOTPSET     = [ 62 0 ];
LOTCPRINT   = [ 63 0 ];
LOTSGRAPH   = [ 64 0 ];
LOTCGRAPH   = [ 65 0 ];
LOTZOOM     = [ 66 0 ];
LOTNSPLIT   = [ 67 0 ];
LOTNSROWS   = [ 68 0 ];
LOTNSCOLS   = [ 69 0 ];
LOTRULER    = [ 70 0 ];
LOTSNRANGE  = [ 71 0 ];
LOTACOMM    = [ 72 0 ];
LOTAMACRO   = [ 73 0 ];
LOTQPARSE   = [ 74 0 ];
LOTWKSPASS  = [ 75 0 ];
LOTHIDCOL2  = [ 101 32 ];
LOTPARSE    = [ 102 16 ];
LOTRRANGES  = [ 103 25 ];
LOTMRANGES  = [ 105 40 ];
else
%
% currently used
%
LOTBEGIN    = [ 0 0 ];
LOTEND      = [ 1 0 ];
LOTCALCMOD  = [ 2 1 ];
LOTCALCORD  = [ 3 1 ];
LOTSPLTWM   = [ 4 1 ];
LOTSPLTWS   = [ 5 1 ];
LOTDIMENSIONS   = [ 6 8 ];
LOTWINDOW1  = [ 7 32 ];
LOTCOLWID1      = [ 8 3 ];
LOTCOLW1        = [ 8 3 ];
LOTCOLW2    = [ 10 3 ];
LOTNMRNG    = [ 11 24 ];
LOTNRANGE   = [ 11 24 ];
LOTBLANK    = [ 12 5 ];
LOTINTEGER  = [ 13 7 ];
LOTNUMBER   = [ 14 13 ];
LOTLABEL    = [ 15 -1 ];
LOTFORMULA  = [ 16 -1 ];
%
LOTMARGINS  = [ 40 10 ];
LOTLABELFMT = [ 41 1 ];
%
LOTCALCCNT  = [ 47 1 ];
%
LOTCURSORW12    = [ 49 1 ];
%
LOTHIDCOL   = [ 100 32 ];
LOTCPI      = [ 150 6 ];
end
%
% constants
%
LOTWK1BOFSTR = [0 0 2 0 6 4];
LOTEOFSTR   = [1 0 0 0];

%
% initializing output
%

m = [];
n = '';
out1 = '';
out2 = '';

% Validate input args
if nargin==0, error('Not enough input arguments.'); end

% Get Filename
if ~isstr(filename), error('Filename must be a string.'); end

% do some validation
if isempty(filename), error('Filename must not be empty.'); end

% put extension on
if all(filename~='.') filename = [filename '.wk1']; end

% Make sure file exists
if ~isequal(exist(filename), 2), error('File not found.'), end

% open the file Lotus uses Little Endian Format ONLY
fid = fopen(filename,'rb', 'l');
if fid == (-1)
    error(['Could not open file ', filename ,'.']);
end

%
% check/set row,col offsets
%
if nargin < 2
    r = 0;
end
if nargin < 3
    c = 0;
end
if nargin < 4
    % max range of cells for WK1 format
    rng = [1 1 8192 256];
end
%
% get the upper-left and bottom-right cells
% of the range to read into MATLAB
%
ulc = []; brc = [];
foundrng = 1;
if ~isstr(rng)
    % user gave us a range, in matlab form, to import [1 1 3 5]
    ulc = rng(1:2);
    brc = rng(3:4);
else
    x = str2rng(rng);
    if ~isempty(x)
        % user gave us a cell range to import 'A1..C5'
        ulc = x(1:2)+1;
        brc = x(3:4)+1;
    else
        foundrng = 0;
    end
end

%
% Flip ulc and brc since we assume ulc = [C R] and brc = [C R] below.
%
if ~isempty(ulc),
  ulc = fliplr(ulc);
  brc = fliplr(brc);
end

%
% Read Lotus WK1 BOF
%
header = fread(fid, 6,'uchar');
if(header(1) ~= LOTWK1BOFSTR)
    error('Not a valid WK1 file.');
end

%
% Start processing WK1 Records
% Read WK1 record header
% cell = [col row]
% Note: Convert Lotus 0 based to 1 based cell coordinates 
%
rec = fread(fid, 2, 'ushort');
while(rec(1) ~= LOTEND(1))
    if(rec(1) == LOTNUMBER(1))
        %
        % 8 byte double
        %
        fmt  = fread(fid, 1,'uchar');
        cell = fread(fid, 2,'ushort');
        cell = cell' + 1;
        val  = fread(fid, 1,'double');
        if(~isempty(ulc) & (cell >= ulc) & (cell <= brc))
            m(cell(2)+r,cell(1)+c) = val(1);
        end
    else
        if(rec(1) == LOTINTEGER(1))
            %
            % 2 byte integer
            %
            fmt  = fread(fid, 1,'uchar');
            cell = fread(fid, 2,'ushort');
            val  = fread(fid, 1,'short');
            cell = cell' + 1;
            if(~isempty(ulc) & (cell >= ulc) & (cell <= brc))
                m(cell(2)+r,cell(1)+c) = val(1);
            end
        else
            if(rec(1) == LOTFORMULA(1))
                %
                % 8 byte double from a Formula
                %
                fmt  = fread(fid, 1,'uchar');
                cell = fread(fid, 2,'ushort');
                cell = cell' + 1;
                val  = fread(fid, 1,'double');
                if((cell >= ulc) & (cell <= brc))
                    m(cell(2)+r,cell(1)+c) = val(1);
                end
                fread(fid, rec(2)-13,'uchar');
            else
                if(rec(1) == LOTNRANGE(1) & isstr(rng))
                    %
                    % Named Range
                    %
                    if isstr(rng) 
                        n = fread(fid, 16,'char');
                        n = setstr(n');
                        n = deblank(n);
                        nrng = fread(fid, 4,'ushort');
                        nrng = nrng';
                        % need to pad n with zeros, this is a bug in strcmp
                        rng = setstr(rng);
                        rng = deblank(rng);
                        if strcmpi(rng,n)
                            foundrng = 1;
                            % found the named range the user wants
                            ulc = nrng(1:2) + 1;
                            brc = nrng(3:4) + 1;
                         end
                    end
                else
                    if(rec(1) == LOTLABEL(1))
                        %
                        % String label
                        %
                        fmt  = fread(fid, 1,'uchar');
                        cell = fread(fid, 2,'ushort');
                        cell = cell' + 1;
                        val  = fread(fid, rec(2) - 5,'uchar');
                        str = char(val');
                        if (str(1) == '''')
                            % shave off trailing null
                            str = str(2:end-1);
                        end
                        if(~isempty(ulc) & (cell >= ulc) & (cell <= brc))
                            n{cell(2)+r,cell(1)+c} = str;
                        end
                    else
                        %
                        % read past this record
                        %
                        fread(fid, rec(2),'uchar');
                    end
                end
            end
        end
    end
    %
    % get the next WK1 record header
    %
    rec = fread(fid, 2, 'ushort');
end

% close file
fclose(fid);

% Return only the valid part
%if exist('m','var')
    m = m(2*r+1:end,2*c+1:end);
%elseif nargout < 2 | foundrng == 0
%    if foundrng == 0
%        error('Invalid range string or unknown named range argument.');
%    else
%        error('No numeric data in WK1 file - use two output args to look for text data.');
%    end
%elseif exist('n','var')
%    m = zeros(size(n));
%else
%    m = [];
%end

%
% remove zero row and column from m
%
row = [];
col = [];

[m1 m2] = size(m);
if (m1>0)&(m2>0);
   for i=1:m1;
      if m(i,:)==zeros(1,m2);
         row = i;
         break;
      end;
   end;
   for i=1:m2;
      if m(:,i)==zeros(m1,1);
         col = i;
         break;
      end;
   end;
   if isempty(col)==0;
      if col==1;
         m = m(:,2:m2);
      elseif col==m2;
         m = m(:,1:m2-1);
      else;
         m = [m(:,1:col-1) m(:,col+1:m2)];
      end;
   end;
   if isempty(row)==0;
      if row==1;
         m = m(2:m1,:);
      elseif row==m1;
         m = m(1:m1-1,:);
      else;
         m = [m(1:row-1,:);m(row+1:m1,:)];
      end;
   end;
end;

% Handle text data
if nargout > 1
%    if exist('n','var')
        n = n(2*r+1:end,2*c+1:end);
%    else
%        n = {};
%    end
end

%
% if n only contains one row, we need to check if it includes the entry date. if it
% does we need to remove the column of m coreesponding to date and enter it into n
%

if (size(n,1)==1)&(size(n,2)>=1);
   b_fix = 0;
   for i=1:size(n,2);
      if strcmp(lower(strtrim(n{1,i})),'date')==1;
         b_fix = i;
         break;
      end;
   end;
   if b_fix>0;
      date_data = m(:,b_fix);
      %
      % check if yearly data
      %
      b_status = ifyearly(date_data);
      if b_status==0;
         if b_fix==1;
            m = m(:,2:size(n,2));
         elseif b_fix==size(n,2);
            m = m(:,1:size(n,2)-1);
         else;
            m = [m(:,1:b_fix-1) m(:,b_fix+1:size(n,2))];
         end;
         %
         % now we need to enter date_data into n. we first create a cell array
         % with empty elements
         %
         new_n = cell_fcn(size(m,1),size(n,2));
         for i=1:size(m,1);
            new_n{i,b_fix} = [num2str(date_data(i,1)) 'Y'];
         end;
         n = [n;new_n];
         col = b_fix;
      end;
   end;
end;

if nargout>1;
   if (size(n,1)>0)&(size(n,2));
      %
      % retrieve 0,1, or 2 text strings from n
      %
      if (isempty(row)==0)&(isempty(col)==0);
         n{row,col} = [];
      end;
      if isempty(row)==0;
         for i=1:m2;
            if isstr(n{row,i})==1;
               out1 = [out1 ' ' n{row,i}];
            end;
         end;
         out1 = out1(1,2:size(out1,2));
      end;
      if isempty(col)==0;
         for i=1:m1;
            if isstr(n{i,col})==1;
               dtstri = n{i,col};
               dtstri = strrep(dtstri,':',' ');
               dtstri = strrep(dtstri,'-',' ');
               dtstri = strrep(dtstri,'#',' ');
               dtstri = strrep(dtstri,'Q',' ');
               dtstri = strrep(dtstri,'M',' ');
               dtstri = strrep(dtstri,'Y','');
               dtstri = strrep(dtstri,'y','');
               dtstri = strrep(dtstri,'q',' ');
               dtstri = strrep(dtstri,'m',' ');
               dtstri = strrep(dtstri,'@',' ');
               dtstri = strrep(dtstri,'(',' ');
               dtstri = strrep(dtstri,')','');
               dtstri = strrep(dtstri,'[',' ');
               dtstri = strrep(dtstri,']','');
               dtstri = strrep(dtstri,'{',' ');
               dtstri = strrep(dtstri,'}','');
               out2 = strvcat(out2,dtstri);
            end;
         end;
      end;
   end;
end;

%
% end of wk1read_fcn.m
%
